/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "StdAfx.h"
#include "EDebug.h"


void AssertFailedLine (LPCSTR pszFile, ULONG Line, LPCSTR pszExpr)
{
	CHAR szMsg[1024];

	StringCchPrintfA (szMsg, _countof (szMsg), "%s\n\nFile %s\nLine %u\n", pszExpr, pszFile, Line);

	::OutputDebugString (_T("----- debug assert -------------------------------------------\n"));
	::OutputDebugStringA (szMsg);
	::OutputDebugString (_T("--------------------------------------------------------------\n"));

	::MessageBoxA (NULL, szMsg, "debug assert", MB_ICONSTOP);
}


void DbgMsg (LPCSTR pszFormat, ...)
{
	CHAR szMsg[1024];
	va_list argList;
	va_start( argList, pszFormat );

	StringCchVPrintfA (szMsg, _countof (szMsg), pszFormat, argList );
	va_end( argList );

	OutputDebugStringA (szMsg);
}








#ifdef _DEBUG_MEMORY

class EMemory
{
public:
	size_t		m_Size;
	LPCSTR		m_pszFileName;
	int			m_Line;
	void*		m_Pointer;

	EMemory*	m_pNext;
	EMemory*	m_pPrev;
};

EMemory* s_pMemoryNext = NULL;



void DebugMemoryDump ()
{
	EMemory*	pMemory = s_pMemoryNext;

	if (pMemory != NULL)
	{
		DbgMsg ("Detected memory leaks!\n");

		while (pMemory)
		{
			DbgMsg ("%s(%u) : 0x%08X, %u bytes\n", pMemory->m_pszFileName, pMemory->m_Line, pMemory + 1, pMemory->m_Size);
			
#if (_DEBUG_MEMORY > 1)

			CHAR		szTmp[256];
			CHAR		szText[256] = {0};

			for (size_t i=0;i<32;i++)
			{
				if (pMemory->m_Size <= i)
				{	break;
				}
				StringCchPrintfA (szTmp, _countof (szTmp), "%02X ", ((PUCHAR)(pMemory + 1))[i]);
				StringCchCatA (szText, _countof (szText), szTmp);
			}
			DbgMsg ("%s\n", szText);

#endif
			pMemory = pMemory->m_pNext;
		}
	}
}

void* __cdecl operator new(size_t Size, LPCSTR pszFileName, int Line)
{
	EMemory*	pMemory = (EMemory*)malloc (sizeof (EMemory) + Size);

	pMemory->m_Size			= Size;
	pMemory->m_pszFileName	= pszFileName;
	pMemory->m_Line			= Line;
	pMemory->m_Pointer		= pMemory+1;
	pMemory->m_pPrev		= NULL;
	pMemory->m_pNext		= s_pMemoryNext;

	if (s_pMemoryNext != NULL)
	{
		s_pMemoryNext->m_pPrev = pMemory;
	}
	s_pMemoryNext = pMemory;

#if (_DEBUG_MEMORY > 1)
	DbgMsg ("%s(%u) : 0x%08X, %u bytes alloc\n", pMemory->m_pszFileName, pMemory->m_Line, pMemory + 1, pMemory->m_Size);
#endif

	return (pMemory + 1);
}

void* __cdecl operator new[](size_t Size, LPCSTR pszFileName, int Line)
{
	return ::operator new(Size, pszFileName, Line);
}

void __cdecl operator delete(void* p, LPCSTR pszFileName, int Line)
{
	::operator delete(p);
}

void __cdecl operator delete[](void* p, LPCSTR pszFileName, int Line)
{
	::operator delete(p);
}

void* __cdecl operator new(size_t Size)
{
	return ::operator new(Size, "Unknown", 0);
}

void __cdecl operator delete(void* p)
{
	EMemory*	pMemory = s_pMemoryNext;

	ASSERT (p != NULL);

	while (pMemory)
	{	
		if (pMemory->m_Pointer == p)
		{
			if (pMemory->m_pPrev == NULL)
			{
				s_pMemoryNext = pMemory->m_pNext;
				if (s_pMemoryNext != NULL)
				{
					s_pMemoryNext->m_pPrev = NULL;
				}
			}
			else
			{
				pMemory->m_pPrev->m_pNext = pMemory->m_pNext;
				if (pMemory->m_pNext != NULL)
				{
					pMemory->m_pNext->m_pPrev = pMemory->m_pPrev;
				}
			}

#if (_DEBUG_MEMORY > 1)
			DbgMsg ("%s(%u) : 0x%08X, %u bytes free\n", pMemory->m_pszFileName, pMemory->m_Line, pMemory + 1, pMemory->m_Size);
#endif
			free (pMemory);
			return;
		}

		pMemory = pMemory->m_pNext;
	}

	free (p);
}

#endif

