/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "StdAfx.h"
#include "EDialog.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


EDialog::EDialog(LPCTSTR lpszTemplateName)
{
	m_lpszTemplateName = lpszTemplateName;
}

EDialog::~EDialog(void)
{
}

INT_PTR EDialog::DlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	EDialog*	pThis = (EDialog*)(LONG_PTR)::GetWindowLongPtr (hWnd, GWLP_USERDATA);
	INT_PTR		Result = 0;

	if (pThis == NULL)
	{
		if ((message != WM_INITDIALOG) || (lParam == 0))
		{
			return 0;
		}
		
		pThis = (EDialog*)lParam;
		pThis->m_hWnd = hWnd;

#pragma warning (disable:4244)
		::SetWindowLongPtr (hWnd, GWLP_USERDATA, (LONG_PTR)pThis);
#pragma warning (default:4244)
	}
	
	if (pThis->OnDlgProc (hWnd, message, wParam, lParam, &Result))
	{
		return Result;
	}

	switch (message)
	{
	case WM_INITDIALOG:
		{
			if (pThis->OnInitDialog ())
			{
				Result = 1;
			}
		}
		break;

	case WM_SIZE:
		{
			pThis->OnSize ((UINT)wParam, LOWORD (lParam), HIWORD (lParam));
		}
		break;

	case WM_NOTIFY:
		{
			NMHDR* pNmHdr = (NMHDR*)lParam;
			if (::IsWindow (pNmHdr->hwndFrom))
			{
				::SendMessage (pNmHdr->hwndFrom, WM_NOTIFY + WM_REFLECT_BASE, wParam, lParam);
			}

			if (!pThis->OnNotify (wParam, lParam, &Result))
			{
			}
		}
		break;

	case WM_NOTIFY + WM_REFLECT_BASE:
		{
			return pThis->OnNotifyRefrection (wParam, lParam, &Result);
		}
		break;

	case WM_COMMAND:
		{
			if (::IsWindow ((HWND) lParam))
			{
				::SendMessage ((HWND) lParam, WM_COMMAND + WM_REFLECT_BASE, wParam, lParam);
			}

			if (pThis->OnCommand (wParam, lParam))
			{	Result = 1;
				break;
			}
		}
		break;

	case WM_COMMAND + WM_REFLECT_BASE:
		{
			return pThis->OnCommandRefrection (wParam, lParam);
		}
		break;

	case WM_PAINT:
		{
			if (pThis->OnPaint ())
			{	Result = 1;
			}
		}
		break;

	case WM_DESTROY:
		{
			pThis->OnDestroy ();
		}
		break;

	case WM_NCDESTROY:
		{
			pThis->OnNcDestroy ();
			
			pThis->m_hWnd = NULL;
		}
		break;

	default:
		{
		}
		break;
	}

	if (Result != 0)
	{
#pragma warning (disable:4244)
		::SetWindowLongPtr (hWnd, DWLP_MSGRESULT, Result);
#pragma warning (default:4244)
	}

	return Result;
}



BOOL EDialog::OnDlgProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam, INT_PTR* pResult)
{

	return FALSE;
}

BOOL EDialog::OnInitDialog ()
{
	return TRUE;
}

BOOL EDialog::OnCommand (WPARAM wParam, LPARAM lParam)
{
	USHORT NotifyCode	= HIWORD(wParam);
	USHORT Id			= LOWORD(wParam);
	HWND hWndCtl		= (HWND)lParam;
			
	switch (NotifyCode)
	{
	case BN_CLICKED:
		{
			switch (Id)
			{
			case IDOK:
			case IDCANCEL:
				{
					::EndDialog (m_hWnd, Id);
				}
				break;
			}
		}
		break;
	}

	return FALSE;
}


BOOL EDialog::OnNotify (WPARAM wParam, LPARAM lParam, INT_PTR* pResult)
{
	return FALSE;
}

BOOL EDialog::OnNotifyRefrection (WPARAM wParam, LPARAM lParam, INT_PTR* pResult)
{
	return FALSE;
}

BOOL EDialog::OnCommandRefrection (WPARAM wParam, LPARAM lParam)
{
	return FALSE;
}

INT_PTR EDialog::DoModal (HWND hParent)
{
	LPCDLGTEMPLATE	pDialogTemplate		= NULL;
	HGLOBAL			hDialogTemplate		= NULL;
	HINSTANCE		hInstance			= GetInstanceHandle();
	HRSRC			hResource			= NULL;
	INT_PTR			Result				= -1;

	hResource = ::FindResource (hInstance, m_lpszTemplateName, RT_DIALOG);
	if (hResource == NULL)
	{	return -1;
	}

	hDialogTemplate = ::LoadResource (hInstance, hResource);
	if (hDialogTemplate == NULL)
	{	return -1;
	}

	pDialogTemplate = (LPCDLGTEMPLATE)::LockResource (hDialogTemplate);
	if (pDialogTemplate != NULL)
	{	
		Result = ::DialogBoxIndirectParam (hInstance, pDialogTemplate, hParent, DlgProc, (LPARAM)this);
	}

	::FreeResource (hDialogTemplate);

	return Result;
}