/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "StdAfx.h"
#include "EFtpServer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

#define DEF_TIMEOUT	30000

EFtpServer::EFtpServer(void)
{
	m_hStop				= ::CreateEvent (NULL, TRUE, FALSE, NULL);
	m_ClientRunning		= 0;
	m_hThread			= NULL;

	::ZeroMemory (&m_WsaOverlap, sizeof (m_WsaOverlap));
}

EFtpServer::~EFtpServer(void)
{
	Stop ();

	ASSERT (m_hThread == NULL);

	if (m_hStop != NULL)
	{
		::CloseHandle (m_hStop);
		m_hStop = NULL;
	}
}

BOOL EFtpServer::Start ()
{
	ULONG	ThreadId;

	Stop ();

	m_hThread	= ::CreateThread (NULL, 0, FtpListenThread, this, 0, &ThreadId);
	
	if (m_hThread == NULL)
	{
		return FALSE;
	}

	return TRUE;
}

BOOL EFtpServer::Stop ()
{
	if (m_hThread != NULL)
	{
		::SetEvent (m_hStop);
		::WaitForSingleObject (m_hThread, INFINITE);
	}
	return TRUE;
}

BOOL EFtpServer::IsRunning ()
{
	if (m_hThread != NULL)
	{
		//if (::WaitForSingleObject (m_hThread, 0) == WAIT_TIMEOUT)
		{	return TRUE;
		}
	}
	return FALSE;
}

LONG EFtpServer::GetClientCount ()
{
	return m_ClientRunning;
}

VOID EFtpServer::ClearUsers ()
{
	m_UserMap.RemoveAll ();
}

VOID EFtpServer::AddUser (LPCSTR pUser, LPCSTR pPass)
{
	m_UserMap.SetAt (pUser, pPass);
}



ULONG WINAPI EFtpServer::FtpListenThread (LPVOID pParam)
{
	EFtpServer*	pThis = (EFtpServer*)pParam;

	pThis->FtpListenThread ();

	return 0;
}

VOID EFtpServer::FtpListenThread ()
{
	SOCKADDR_IN			Sa				= {0};
	INT					SaLength		= sizeof (Sa);
	int					Value			= 0;
	HANDLE				Handles[2]		= {0};
	ULONG				Result			= 0;
	SOCKET				hClientSocket	= NULL;
	WSANETWORKEVENTS	NetworkEvents	= {0};
	EFtpClientCfg*		pCfg			= NULL;
	SOCKET				hListenSocket	= INVALID_SOCKET;
	INT					Error			= 0;

	do
	{
		Handles[0] = m_hStop;
		Handles[1] = ::WSACreateEvent ();

		hListenSocket = ::WSASocket (AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
		if (hListenSocket == INVALID_SOCKET)
		{	
			break;
		}

		// Bind the socket -------------------------------------------------------------
		Sa.sin_family		= AF_INET;
		Sa.sin_addr.s_addr	= INADDR_ANY;
		Sa.sin_port			= ::htons (21);

		if (::bind (hListenSocket, (PSOCKADDR) &Sa, sizeof (Sa)) == SOCKET_ERROR)
		{
			Error = ::WSAGetLastError ();
			break;
		}

		if (::listen (hListenSocket, SOMAXCONN) == SOCKET_ERROR)
		{
			Error = ::WSAGetLastError ();
			break;
		}

		if (::WSAEventSelect (hListenSocket, Handles[1], FD_ACCEPT) == SOCKET_ERROR)
		{
			Error = ::WSAGetLastError ();
			break;
		}

		while (1)
		{
			Result = ::WSAWaitForMultipleEvents (2, Handles, FALSE, INFINITE, FALSE);
			
			if (Result == (WSA_WAIT_EVENT_0 + 0))//m_hStop
			{	
				break;
			}

			if (Result != (WSA_WAIT_EVENT_0 + 1))//FD_ACCEPT Event
			{	
				break;
			}

			if (::WSAEnumNetworkEvents (hListenSocket, Handles[1], &NetworkEvents) == SOCKET_ERROR)
			{
				break;
			}

			if (!(NetworkEvents.lNetworkEvents & FD_ACCEPT))
			{	continue;
			}

			if (NetworkEvents.iErrorCode[FD_ACCEPT_BIT] != 0)
			{	continue;
			}

			hClientSocket = ::WSAAccept (hListenSocket, (sockaddr*)&Sa, &SaLength, NULL, 0);
			if (hClientSocket == INVALID_SOCKET)
			{	continue;
			}

			pCfg = new EFtpClientCfg;
			if (pCfg == NULL)
			{	continue;
			}

			pCfg->m_pServer = this;
			pCfg->m_hStop	= m_hStop;
			pCfg->m_Main.m_hSocket	= hClientSocket;

			pCfg->m_hThread	= ::CreateThread (NULL, 0, FtpClientThread, pCfg, 0, &pCfg->m_ThreadId);
			
			if (pCfg->m_hThread == NULL)
			{
				::closesocket (pCfg->m_Main.m_hSocket);
				delete pCfg;
			}

		}

	}while(0);

	while (m_ClientRunning != 0)
	{
		::Sleep (100);
	}

	if (hListenSocket != INVALID_SOCKET)
	{
		::closesocket (hListenSocket);
		hListenSocket = INVALID_SOCKET;
	}

	if (Handles[1] != NULL)
	{
		::WSACloseEvent (Handles[1]);
	}

	::CloseHandle (m_hThread);
	m_hThread = NULL;
	::ResetEvent (m_hStop);

}

ULONG WINAPI EFtpServer::FtpClientThread (LPVOID lpThreadParameter)
{
	EFtpClientCfg*	pCfg = (EFtpClientCfg*)lpThreadParameter;

	pCfg->m_pServer->FtpClientThread (pCfg);

	return 0;
}

VOID EFtpServer::FtpClientThread (EFtpClientCfg* pCfg)
{
	INT				SaLength		= sizeof (pCfg->m_Peer);
	PUCHAR			pBuffer			= NULL;
	ULONG			BufferLength	= 64 * 1024;
	ULONG			RecvLength;

	::InterlockedIncrement (&m_ClientRunning);

	do
	{
		pBuffer = new UCHAR[BufferLength];
		if (pBuffer == NULL)
		{	break;
		}

		if(::getpeername (pCfg->m_Main.m_hSocket, (PSOCKADDR) &pCfg->m_Peer, &SaLength) == SOCKET_ERROR)
		{	break;
		}


		OnConnect (pCfg, &pCfg->m_Peer);

		DBG_MSG(("CLIENT START (%s, %u)\r\n", ::inet_ntoa (pCfg->m_Peer.sin_addr), ::htons (pCfg->m_Peer.sin_port)));
		if (::WSAEventSelect (pCfg->m_Main.m_hSocket, pCfg->m_Main.m_hEvent, FD_CLOSE | FD_READ) == SOCKET_ERROR)
		{
		}		

		while (Receive (pCfg, pCfg->m_Main.m_hSocket, pCfg->m_Main.m_hEvent, pBuffer, BufferLength, &RecvLength, DEF_TIMEOUT))
		{
			pBuffer[RecvLength] = 0;
			ParseCommandLine (pCfg, pBuffer);
		}
		
		if (::WSAEventSelect (pCfg->m_Main.m_hSocket, pCfg->m_Main.m_hEvent, 0) == SOCKET_ERROR)
		{	
		}

		DestroyDataConnection (pCfg);
		DBG_MSG(("CLIENT END   (%s, %u)\r\n", ::inet_ntoa (pCfg->m_Peer.sin_addr), ::htons (pCfg->m_Peer.sin_port)));
	
		OnDisconnect (pCfg, &pCfg->m_Peer);

	}
	while (0);

	if (pCfg->m_Main.m_hSocket != INVALID_SOCKET)
	{
		::closesocket (pCfg->m_Main.m_hSocket);
		pCfg->m_Main.m_hSocket = INVALID_SOCKET;
	}

	if (pBuffer != NULL)
	{
		delete pBuffer;
	}

	::InterlockedDecrement (&m_ClientRunning);

	delete pCfg;
}

VOID EFtpServer::OnConnect (EFtpClientCfg* pCfg, SOCKADDR_IN* pAddr)
{
	SendResponse (pCfg, "220 Virtual FTP Server (%s, %u)\r\n", ::inet_ntoa (pAddr->sin_addr), ::htons (pAddr->sin_port));
}

VOID EFtpServer::OnDisconnect (EFtpClientCfg* pCfg, SOCKADDR_IN* pAddr)
{
}

BOOL EFtpServer::SendResponse(EFtpClientCfg* pCfg, LPCSTR pstrFormat, ...)
{
	CHAR szResponse[4 * 1024] = {0};
	size_t SendLength = 0;

	va_list args;
	va_start(args, pstrFormat);
	StringCchVPrintfA (szResponse, _countof (szResponse), pstrFormat, args);

	if (FAILED (StringCchLengthA (szResponse, _countof (szResponse), &SendLength)))
	{	return FALSE;
	}

	return Send (pCfg, pCfg->m_Main.m_hSocket, szResponse, SendLength, DEF_TIMEOUT);
}

BOOL EFtpServer::Send (EFtpClientCfg* pCfg, SOCKET hSocket, PVOID pBuffer, size_t SendLength, ULONG Timeout)
{
	SOCKADDR_IN		FromAddress		= {0};
	INT				FromLength		= sizeof (FromAddress);
	WSABUF			SendDesc		= {0};
	ULONG			BytesSent		= 0;
	WSAOVERLAPPED	WsaOverlap		= {0};
	DWORD			Flags			= 0;
	DWORD			Result			= 0;
	INT				Error			= ERROR_SUCCESS;
	BOOL			RetCode			= FALSE;
	HANDLE			Handles[2]		= {0};



	Handles[0] = pCfg->m_hStop;
	Handles[1] = WSACreateEvent ();

	SendDesc.buf		= (PCHAR)pBuffer;
	SendDesc.len		= (ULONG)SendLength;
	BytesSent			= SendDesc.len;
	WsaOverlap.hEvent	= Handles[1];

	if (::WSASend (hSocket, &SendDesc, 1, &BytesSent, 0, &WsaOverlap, NULL) == SOCKET_ERROR)
	{
		BytesSent = 0;
		Error = ::WSAGetLastError ();
		if (Error == WSA_IO_PENDING)
		{	
				DBG_MSG(("EFtpServer::Send - WSA_IO_PENDING, pCfg=%p \r\n", pCfg));
			Result = ::WSAWaitForMultipleEvents (2, Handles, FALSE, Timeout, FALSE);
			if (Result == WAIT_TIMEOUT)
			{
				::CancelIo ((HANDLE)hSocket);
				//RetCode = TRUE;
				DBG_MSG(("EFtpServer::Send - CancelIo, pCfg=%p \r\n", pCfg));

			}
			else if (Result == (WAIT_OBJECT_0 + 0))
			{
				DBG_MSG(("EFtpServer::Send - STOP, pCfg=%p \r\n", pCfg));

			}
			else if (Result == (WAIT_OBJECT_0 + 1))
			{
				DBG_MSG(("EFtpServer::Send - EVENT, pCfg=%p \r\n", pCfg));

				if (::WSAGetOverlappedResult (hSocket, &WsaOverlap, &BytesSent, TRUE, &Flags))
				{
					RetCode = TRUE;
				}
			}
		}
	}
	else
	{
		RetCode = TRUE;
	}

	if (Handles[1] != NULL)
	{
		::WSACloseEvent (Handles[1]);
	}

	return RetCode;
}

BOOL EFtpServer::Receive (EFtpClientCfg* pCfg, SOCKET hSocket, HANDLE hEvent, PVOID pBuffer, size_t BufferLength, PULONG pBytesReceive, ULONG Timeout)
{
	SOCKADDR_IN			FromAddress		= {0};
	INT					FromLength		= sizeof (FromAddress);
	WSABUF				RecvDesc		= {0};
	ULONG				BytesRecv		= 0;
	DWORD				Flags			= 0;
	DWORD				RetCode1		= 0;
	DWORD				RetCode2		= 0;
	INT					Error			= 0;
	HANDLE				hEventReceive	= NULL;
	WSANETWORKEVENTS	NetworkEvents	= {0};
	INT					Result			= 0;
	BOOL				RetCode			= FALSE;
	HANDLE				Handles[2]		= {0};
	WSAOVERLAPPED		WsaOverlap		= {0};


	do
	{
		Handles[0] = pCfg->m_hStop;
		Handles[1] = hEvent;

		Result = ::WSAWaitForMultipleEvents (2, Handles, FALSE, Timeout, FALSE);
		if (Result == WSA_WAIT_TIMEOUT)
		{	
			DBG_MSG(("EFtpServer::Receive - WSA_WAIT_TIMEOUT, pCfg=%p \r\n", pCfg));
			RetCode = TRUE;
			break;
		}
		if (Result == (WSA_WAIT_EVENT_0 + 0))
		{	
			DBG_MSG(("EFtpServer::Receive - m_hStop, pCfg=%p \r\n", pCfg));
			break;
		}

		if (::WSAEnumNetworkEvents (hSocket, hEvent, &NetworkEvents) == SOCKET_ERROR)
		{
			DBG_MSG(("EFtpServer::Receive - WSAEnumNetworkEvents failure, pCfg=%p \r\n", pCfg));
			break;
		}
		
		if (NetworkEvents.lNetworkEvents == 0)
		{
			DBG_MSG(("EFtpServer::Receive - NetworkEvents.lNetworkEvents==0, pCfg=%p \r\n", pCfg));
			RetCode = TRUE;
			break;
		}

		if ((NetworkEvents.lNetworkEvents & FD_READ) && (NetworkEvents.iErrorCode[FD_READ_BIT] == 0))
		{
			RecvDesc.buf		= (PCHAR)pBuffer;
			RecvDesc.len		= (ULONG)BufferLength;
			BytesRecv			= 0;
			Flags				= 0;

			WsaOverlap.hEvent	= ::WSACreateEvent ();

			if (::WSARecvFrom (hSocket, &RecvDesc, 1, &BytesRecv, &Flags, (sockaddr*)&FromAddress, &FromLength,  &WsaOverlap, NULL) != SOCKET_ERROR)
			{	
				RetCode = TRUE;
			}
			else if (::WSAGetLastError () == ERROR_IO_PENDING)
			{
				Result = ::WSAWaitForMultipleEvents (1, &WsaOverlap.hEvent, FALSE, Timeout, FALSE);
				if (Result == WAIT_TIMEOUT)
				{
					CancelIo ((HANDLE)hSocket);
					RetCode = TRUE;
					DBG_MSG(("EFtpServer::Receive - CancelIo, pCfg=%p \r\n", pCfg));

				}
				else
				{
					if (::WSAGetOverlappedResult (hSocket, &WsaOverlap, &BytesRecv, TRUE, &Flags))
					{
						RetCode = TRUE;
					}
					else
					{
						DBG_MSG(("EFtpServer::Receive - WSAGetOverlappedResult failure, pCfg=%p \r\n", pCfg));
					}
				}
			}
			else
			{
				Result = ::WSAGetLastError ();
				DBG_MSG(("EFtpServer::Receive - WSARecvFrom failure, pCfg=%p, Result=%08X \r\n", pCfg, Result));
			}

			if (WsaOverlap.hEvent != NULL)
			{
				::WSACloseEvent (WsaOverlap.hEvent);
			}
		}

		if (NetworkEvents.lNetworkEvents & FD_CLOSE)
		{
			DBG_MSG(("EFtpServer::Receive - FD_CLOSE, pCfg=%p \r\n", pCfg));
			break;
		}
	}
	while(0);
	
	if (pBytesReceive != NULL)
	{	*pBytesReceive = BytesRecv;
	}

	return RetCode;
}

VOID EFtpServer::ParseCommandLine (EFtpClientCfg* pCfg, PUCHAR pBuffer)
{
	CHAR			szCommand[64];
	CHAR			szArguments[1024];
	CHAR			szTemp[1024];
	PCHAR			pszRxBuffer;
	PCHAR			pszFound;
	size_t			Length;

//	DBG_MSG(("EFtpServer::ParseCommandLine - pCfg=%p \r\n", pCfg));

	pszRxBuffer = (PCHAR)pBuffer;

	while (*pszRxBuffer)
	{
		pszFound = StringCchStrStrA (pszRxBuffer, STRSAFE_MAX_CCH, "\r\n");
		if (pszFound == NULL)
		{	break;
		}

		StringCchCopyNA (szTemp, _countof (szTemp), pszRxBuffer, pszFound - pszRxBuffer);
		pszRxBuffer = pszFound + 2;

		if (szTemp[0] == 0)
		{	continue;
		}

		pszFound = StringCchStrStrA (szTemp, _countof (szTemp), " ");

		if (pszFound != NULL)
		{
			StringCchCopyNA (szCommand, _countof (szCommand), szTemp, pszFound - szTemp);
			StringCchCopyNA (szArguments, _countof (szArguments), pszFound + 1, _countof (szArguments));
		}
		else
		{
			StringCchCopyNA (szCommand, _countof (szCommand), szTemp, _countof (szCommand));
			StringCchCopyNA (szArguments, _countof (szArguments), "", _countof (szCommand));
		}

		if (szCommand[0] == 0)
		{	continue;
		}

		if (FAILED (StringCchLengthA (szCommand, _countof (szCommand), &Length)))
		{	continue;
		}

		for (size_t i=0;i<Length;i++)
		{
			if ((szCommand[i] >= 'a') && (szCommand[i] <= 'z'))
			{
				szCommand[i] = szCommand[i] - 'a' + 'A';
			}
		}

		if (Length > 4)
		{
			if (::CompareStringA (LOCALE_USER_DEFAULT, 0, &szCommand[Length - 4], 4, "ABOR", 4) == CSTR_EQUAL)
			{
				StringCchCopyNA (szCommand, _countof (szCommand), "ABOR", _countof (szCommand));
			}
		}

		ProcessCommand (pCfg, szCommand, szArguments);
	}
}


VOID EFtpServer::ProcessCommand (EFtpClientCfg* pCfg, LPCSTR Cmd, LPCSTR Arg)
{
	DBG_MSG(("EFtpServer::ProcessCommand - pCfg=%p, %s, %s \r\n", pCfg, Cmd, Arg));

	if ((lstrcmpA (Cmd, "BYE") != 0) && (lstrcmpA (Cmd, "QUIT") != 0) && (lstrcmpA (Cmd, "USER") != 0) && (lstrcmpA(Cmd, "PASS") != 0) && (pCfg->m_nStatus != STATUS_IDLE))
	{
		SendResponse (pCfg, "530 Please login with USER and PASS.\r\n");
		return;
	}

		 if (lstrcmpA (Cmd, "ABOR") == 0)	{														ProcessABOR (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "APPE") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessSTOR (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "BYE") == 0)	{														ProcessQUIT (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "CDUP") == 0)	{														ProcessCWD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "CWD") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessCWD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "DELE") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessDELE (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "HELP") == 0)	{														ProcessHELP (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "LIST") == 0)	{														ProcessLIST (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "MKD") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessMKD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "NLST") == 0)	{														ProcessLIST (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "NOOP") == 0)	{														ProcessNOOP (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "PASS") == 0)	{														ProcessPASS (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "PASV") == 0)	{														ProcessPASV (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "P@SW") == 0)	{														ProcessPASV (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "PORT") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessPORT (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "PWD") == 0)	{														ProcessPWD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "QUIT") == 0)	{														ProcessQUIT (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "REST") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessREST (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "RETR") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessRETR (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "RMD") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessRMD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "RNFR") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessRNFR (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "RNTO") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessRNTO (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "SITE") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessSITE (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "SIZE") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessSIZE (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "STOR") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessSTOR (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "SYST") == 0)	{														ProcessSYST (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "TYPE") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessTYPE (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "USER") == 0)	{														ProcessUSER (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "XCWD") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessCWD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "XMKD") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessMKD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "XPWD") == 0)	{														ProcessPWD  (pCfg, Cmd, Arg);}
	else if (lstrcmpA (Cmd, "XRMD") == 0)	{	if (Arg[0] == 0){SendSyntaxError (pCfg); return; }	ProcessRMD  (pCfg, Cmd, Arg);}
	else { 	SendResponse (pCfg, "502 Command \"%s\" not implemented - Try HELP.\r\n", Cmd);}
			 
}			 


VOID EFtpServer::SendSyntaxError (EFtpClientCfg* pCfg)
{
	SendResponse (pCfg, "501 Syntax error: Invalid number of parameters.\r\n");
}


// Abort transfer
VOID EFtpServer::ProcessABOR (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("EFtpServer::ProcessABOR - pCfg=%p \r\n", pCfg));

	if (pCfg->m_Data.m_hSocket != NULL)
	{
		if (pCfg->m_nStatus != STATUS_IDLE)
		{
			SendResponse (pCfg, "426 Data connection closed.\r\n");
		}

		DestroyDataConnection(pCfg);
	}
	SendResponse (pCfg, "226 ABOR command successful.\r\n");
}

// Change working directory: CWD [directory-name]
// Change to parent directory: CDUP
// Change working directory: XCWD [directory-name]
VOID EFtpServer::ProcessCWD (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{

	if (lstrcmpA (pszArguments, "/") == 0)
	{
		SendResponse (pCfg, "250 \"%s\" is current directory.\r\n", pCfg->m_szCurrentDir);
		return;
	}
	SendResponse (pCfg, "550 \"%s\": Directory not found.\r\n", pszArguments);
}

// Delete file: DELE file-name
VOID EFtpServer::ProcessDELE (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "550 Permission denied.\r\n");
}

// Show help: HELP [command]
VOID EFtpServer::ProcessHELP (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	if (lstrcmpA (pszArguments, "") == 0)
	{
		CHAR szResponse[] = 
			"214-The following commands are recognized:\r\n"
			"ABOR\tAPPE\tBYE\tCDUP\r\n"
			"CWD\tDELE\tHELP\tLIST\r\n"
			"MKD\tNLST\tNOOP\tPASS\r\n"
			"PASV\tP@SW\tPORT\tPWD\r\n"
			"QUIT\tREST\tRETR\tRMD\r\n"
			"RNFR\tRNTO\tSITE\tSIZE\r\n"
			"STOR\tSYST\tTYPE\tUSER\r\n"
			"XCWD\tXMKD\tXPWD\tXRMD\r\n"
			"214 HELP command successful.\r\n";

		SendResponse (pCfg, szResponse);
	}
	else
	{
		SendResponse (pCfg, "501 Unknown command %s\r\n", pszArguments);
	}
}

// Get directory listing: LIST [path-name] 
// List filenames only: NLST [path-name]
VOID EFtpServer::ProcessLIST (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	ULONG BufferLength = 64 * 1024;
	PCHAR pszListing = new CHAR[BufferLength];
	size_t Length = 0;

	DBG_MSG(("EFtpServer::ProcessLIST - pCfg=%p \r\n", pCfg));

	if (pszListing == NULL)
	{
		SendResponse (pCfg, "550 System memory error.\r\n");
		return;
	}

	::ZeroMemory (pszListing, BufferLength);

	if (lstrcmpA (pszCommand, "LIST") == 0)
	{
		//wsprintfA (pszListing, "-r--r--r--   1 aaaa aaaa       10485760 Jan  1  2003 Readme.txt\r\n");
	}else
	{
		//wsprintfA (pszListing, "Readme.txt\r\n");
	}

	SendResponse (pCfg, "150 Opening ASCII mode data connection for directory list.\r\n"); 
	
	// create data connection with client
	if (!CreateDataConnection(pCfg))
	{
		SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");//?
		return;
	}

	pCfg->m_nStatus = STATUS_LIST;
		
	if (FAILED (StringCchLengthA (pszListing, BufferLength, &Length)))
	{	
		SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");//?
		return;
	}

	// send the listing
	Send (pCfg, pCfg->m_Data.m_hSocket, pszListing, Length, DEF_TIMEOUT);

	DestroyDataConnection (pCfg);
	SendResponse (pCfg, "226 Transfer complete.\r\n");
	pCfg->m_nStatus = STATUS_IDLE;
}

// Make directory: MKD path-name
// Make directory: XMKD path-name
VOID EFtpServer::ProcessMKD (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("EFtpServer::ProcessMKD - pCfg=%p \r\n", pCfg));
	SendResponse (pCfg, "550 Can't create directory. Permission denied.\r\n");
}

// Do nothing: NOOP
VOID EFtpServer::ProcessNOOP (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("EFtpServer::ProcessNOOP - pCfg=%p \r\n", pCfg));

	SendResponse (pCfg, "200 OK\r\n");
}

// Supply a username: USER username
VOID EFtpServer::ProcessUSER (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("EFtpServer::ProcessUSER - pCfg=%p \r\n", pCfg));

	if ((pCfg->m_nStatus != STATUS_LOGIN) && (pCfg->m_nStatus != STATUS_LOGIN_USER))
	{
		SendResponse (pCfg, "530 Not in authorization state\r\n");
		return;
	}

	//if (m_UserMap.IsEmpty ())
	//{
	//	StringCchCopyA (pCfg->m_szCurrentDir, _countof (pCfg->m_szCurrentDir), "/");
	//	pCfg->m_nStatus = STATUS_IDLE;
	//	SendResponse (pCfg, "230 User successfully logged in.\r\n");
	//	return;
	//}

	pCfg->m_nStatus	= STATUS_LOGIN_USER;
	StringCchCopyNA (pCfg->m_szUserName, _countof (pCfg->m_szUserName), pszArguments, _countof (pCfg->m_szUserName));
	SendResponse(pCfg, "331 Password required for %s\r\n", pszArguments);
}

// Supply a user password: PASS password
VOID EFtpServer::ProcessPASS (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("EFtpServer::ProcessPASS - pCfg=%p \r\n", pCfg));

	if (pCfg->m_nStatus	!= STATUS_LOGIN_USER)
	{
		SendResponse (pCfg, "503 Login with USER first.\r\n");
		pCfg->m_nStatus	= STATUS_LOGIN;
		return;
	}

	if (!m_UserMap.IsEmpty ())
	{
		LPCSTR pszPass;

		if (!m_UserMap.Lookup (pCfg->m_szUserName, pszPass))
		{
			SendResponse (pCfg, "530 Invalid userid/password.\r\n");
			pCfg->m_nStatus	= STATUS_LOGIN;
			return;
		}

		if (lstrcmpA (pszPass, pszArguments) != 0)
		{
			SendResponse (pCfg, "530 Invalid userid/password.\r\n");
			pCfg->m_nStatus	= STATUS_LOGIN;
			return;
		}
	}

	StringCchCopyA (pCfg->m_szCurrentDir, _countof (pCfg->m_szCurrentDir), "/");
	pCfg->m_nStatus = STATUS_IDLE;
	SendResponse (pCfg, "230 User successfully logged in.\r\n");
}

// Set server in passive mode: PASV
// Set server in passive mode: P@SW
VOID EFtpServer::ProcessPASV (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("EFtpServer::ProcessPASV - pCfg=%p \r\n", pCfg));

	pCfg->m_bPassiveMode = TRUE;

	if (!CreatePasvDataConnection (pCfg))
	{	return;
	}
}

// Specify the client port number: PORT a0,a1,a2,a3,a4,a5
VOID EFtpServer::ProcessPORT (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	int				Number[6] = {0};
	int				Index = 0;
	LPCSTR			p = pszArguments;

	DBG_MSG(("EFtpServer::ProcessPORT - pCfg=%p \r\n", pCfg));

	while (*p)
	{
		if ((*p < '0') || (*p > '9'))
		{	p++;
			continue;
		}

		Number[Index++] = atoi (p);

		while ((*p >= '0') && (*p <= '9'))
		{	p++;
		}
	}

	pCfg->m_RemoteAddress = MAKELONG (MAKEWORD (Number[0], Number[1]), MAKEWORD (Number[2], Number[3]));
	pCfg->m_RemotePort	= 256 * Number[4] + Number[5];
	pCfg->m_bPassiveMode	= FALSE;
	SendResponse (pCfg, "200 Port command successful.\r\n");
}

// Get current directory: PWD
// Get current directory: XPWD
VOID EFtpServer::ProcessPWD (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "257 \"%s\" is current directory.\r\n", pCfg->m_szCurrentDir);
}

// Logout or break the connection: QUIT
// Logout or break the connection: BYE
VOID EFtpServer::ProcessQUIT (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("EFtpServer::ProcessQUIT - pCfg=%p \r\n", pCfg));

	SendResponse (pCfg, "221 Goodbye.\r\n");
	DestroyDataConnection (pCfg);

	::closesocket (pCfg->m_Main.m_hSocket);
	pCfg->m_Main.m_hSocket = INVALID_SOCKET;

}

// Set restart transfer marker: REST marker
VOID EFtpServer::ProcessREST (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "350 Restarting at %d.\r\n", atol (pszArguments));
}

// Get file: RETR file-name
VOID EFtpServer::ProcessRETR (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DestroyDataConnection (pCfg);
	SendResponse (pCfg, "550 File not found.\r\n");
}


// Remove directory: RMD path-name
// Remove directory: XRMD path-name
VOID EFtpServer::ProcessRMD (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "550 Permission denied.\r\n");
}

// Specify old path name of file to be renamed: RNFR file-name
VOID EFtpServer::ProcessRNFR (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "550 Permission denied.\r\n");
}

// Specify new path name of file to be renamed: RNTO file-name
VOID EFtpServer::ProcessRNTO (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "550 Permission denied.\r\n");
}

// Change Site Attributes: SITE [MODE] [file-name]
VOID EFtpServer::ProcessSITE (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "550 Permission denied.\r\n");
}

// Get filesize: SIZE file-name
VOID EFtpServer::ProcessSIZE (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "550 Permission denied.\r\n");
}

// Store file: STOR file-name
// Append file to existing file: APPE file-name 
VOID EFtpServer::ProcessSTOR (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	PUCHAR		pBuffer			= NULL;
	ULONG		BufferLength	= 64 * 1024;
	ULONG		RecvLength;
	ULONG		StartTime;

	DBG_MSG(("EFtpServer::ProcessSTOR - pCfg=%p, %s, %s \r\n", pCfg, pszCommand, pszArguments));


	SendResponse (pCfg, "150 Opening BINARY mode data connection for file transfer.\r\n");

	if (!CreateDataConnection(pCfg))
	{
		pCfg->m_nStatus = STATUS_IDLE;
		SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");//?
		return;
	}

	pBuffer = new UCHAR[BufferLength];

	StartTime = ::GetTickCount ();

	while (1)
	{
		if (!Receive (pCfg, pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, pBuffer, BufferLength, &RecvLength, 5000))
		{	break;
		}

		if (RecvLength != 0)
		{	StartTime = ::GetTickCount ();
		}

		if ((::GetTickCount () - StartTime) > 30000)
		{	break;
		}
	};

	if (pBuffer != NULL)
	{	delete pBuffer;
	}

	DestroyDataConnection (pCfg);

	SendResponse (pCfg, "226 Transfer complete.\r\n");
}


// Get operating system type: SYST
VOID EFtpServer::ProcessSYST (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	SendResponse (pCfg, "215 UNIX emulated by Virtual FTP Server.\r\n");
}

// Set filetype: TYPE [A | I]
VOID EFtpServer::ProcessTYPE (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	// let's pretend we did something...
	SendResponse (pCfg, "200 Type set to %s\r\n", pszArguments);
}




BOOL EFtpServer::CreatePasvDataConnection(EFtpClientCfg* pCfg)
{
	SOCKADDR_IN			Sa				= {0};
	INT					SaLength		= sizeof (Sa);
	WSANETWORKEVENTS	NetworkEvents	= {0};
	BOOL				RetCode			= FALSE;
	HANDLE				Handles[2]		= {0};
	SOCKET				hSocket			= INVALID_SOCKET;
	ULONG				Result			= 0;

	DBG_MSG(("EFtpServer::CreatePasvDataConnection - pCfg=%p \r\n", pCfg));

	DestroyDataConnection (pCfg);

	if (!pCfg->m_bPassiveMode)
	{	return FALSE;
	}

	do
	{
		Handles[0] = pCfg->m_hStop;
		Handles[1] = ::WSACreateEvent ();

		hSocket = ::WSASocket (AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
		if (hSocket == INVALID_SOCKET)
		{	break;
		}

		// Bind the socket -------------------------------------------------------------
		Sa.sin_family		= AF_INET;
		Sa.sin_addr.s_addr	= INADDR_ANY;
		Sa.sin_port			= ::htons (INADDR_ANY);

		if (::bind (hSocket, (PSOCKADDR) &Sa, sizeof (Sa)) == SOCKET_ERROR)
		{	break;
		}

		if (::listen (hSocket, 1) == SOCKET_ERROR)
		{	break;
		}

		if (::WSAEventSelect (hSocket, Handles[1], FD_ACCEPT) == SOCKET_ERROR)
		{	break;
		}

		{
			USHORT Port;
			CHAR szIPAddress[128] = {0};
			PCHAR p;

			if (::getsockname (hSocket, (PSOCKADDR) &Sa, &SaLength) == SOCKET_ERROR)
			{
				SendResponse (pCfg, "421 Failed to create socket.\r\n");
				return FALSE;
			}

			Port = ::htons (Sa.sin_port);

			if (::getsockname (pCfg->m_Main.m_hSocket, (PSOCKADDR) &Sa, &SaLength) == SOCKET_ERROR)
			{
				SendResponse (pCfg, "421 Failed to create socket.\r\n");
				return FALSE;
			}

			StringCchCopyNA (szIPAddress, _countof (szIPAddress), ::inet_ntoa (Sa.sin_addr), _countof (szIPAddress));
			
			p = szIPAddress;
			while (*p)
			{
				if (*p == '.')
				{
					*p = ',';
				}
				p++;
			}

			SendResponse (pCfg, "227 Entering Passive Mode (%s,%d,%d).\r\n", szIPAddress, Port / 256, Port % 256);
		}

		Result = ::WSAWaitForMultipleEvents (2, Handles, FALSE, INFINITE, FALSE);
		if (Result == (WSA_WAIT_EVENT_0 + 0))
		{	break;
		}

		if (::WSAEnumNetworkEvents (hSocket, Handles[1], &NetworkEvents) == SOCKET_ERROR)
		{	break;
		}

		if (!(NetworkEvents.lNetworkEvents & FD_ACCEPT))
		{	break;
		}

		if (NetworkEvents.iErrorCode[FD_CONNECT_BIT] != 0)
		{	break;
		}

		pCfg->m_Data.m_hSocket = ::WSAAccept (hSocket, (sockaddr*)&Sa, &SaLength, NULL, 0);
		if (pCfg->m_Data.m_hSocket == INVALID_SOCKET)
		{	break;
		}

		int RcvBuf = 0;
		int RcvBufLen = sizeof (RcvBuf);
		::getsockopt (pCfg->m_Data.m_hSocket, SOL_SOCKET, SO_RCVBUF, (char*)&RcvBuf, &RcvBufLen);
		RcvBuf = 64 * 1024 -1;
		::setsockopt (pCfg->m_Data.m_hSocket, SOL_SOCKET, SO_RCVBUF, (char*)&RcvBuf, sizeof (RcvBuf));
		::getsockopt (pCfg->m_Data.m_hSocket, SOL_SOCKET, SO_RCVBUF, (char*)&RcvBuf, &RcvBufLen);

		if (::WSAEventSelect (pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, FD_CLOSE | FD_READ) == SOCKET_ERROR)
		{
		}

		RetCode = TRUE;


	}while(0);

	if (Handles[1] != NULL)
	{
		if (::WSAEventSelect (hSocket, Handles[1], 0) == SOCKET_ERROR)
		{	
		}
		::WSACloseEvent (Handles[1]);
	}

	return RetCode;
}

BOOL EFtpServer::CreateDataConnection(EFtpClientCfg* pCfg)
{
	SOCKADDR_IN			Sa				= {0};
	INT					SaLength		= sizeof (Sa);
	WSANETWORKEVENTS	NetworkEvents	= {0};
	BOOL				RetCode			= FALSE;
	HANDLE				Handles[2]		= {0};
	SOCKET				hSocket			= INVALID_SOCKET;
	ULONG				Result			= 0;

	DBG_MSG(("EFtpServer::CreateDataConnection - pCfg=%p \r\n", pCfg));

	if (pCfg->m_bPassiveMode)
	{	return (pCfg->m_Data.m_hSocket != INVALID_SOCKET);
	}

	DestroyDataConnection (pCfg);

	// need PORT command first !
	if ((pCfg->m_RemoteAddress == 0) || (pCfg->m_RemotePort == 0))
	{
		SendResponse(pCfg, "425 Can't open data connection.\r\n");
		return FALSE;
	}

	do
	{
		pCfg->m_Data.m_hSocket = ::WSASocket (AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, 0, WSA_FLAG_OVERLAPPED);
		if (pCfg->m_Data.m_hSocket == INVALID_SOCKET)
		{	break;
		}

		// Bind the socket -------------------------------------------------------------
		Sa.sin_family		= AF_INET;
		Sa.sin_addr.s_addr	= INADDR_ANY;
		Sa.sin_port			= ::htons (20);

		if (::bind (pCfg->m_Data.m_hSocket, (PSOCKADDR) &Sa, sizeof (Sa)) == SOCKET_ERROR)
		{
			break;
		}

		// connect the socket -------------------------------------------------------------
		Sa.sin_family		= AF_INET;
		Sa.sin_addr.s_addr	= pCfg->m_RemoteAddress;
		Sa.sin_port			= ::htons (pCfg->m_RemotePort);


		Handles[0]	= pCfg->m_hStop;
		Handles[1]	= pCfg->m_Data.m_hEvent;
		if (Handles[1] == NULL)
		{	break;
		}

		//񓯊[h (ڑ)
		if (::WSAEventSelect (pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, FD_CONNECT) == SOCKET_ERROR)
		{	break;
		}

		if (::WSAConnect (pCfg->m_Data.m_hSocket, (SOCKADDR*)&Sa, sizeof (Sa), NULL, NULL, NULL, NULL) == SOCKET_ERROR)
		{
			if (::WSAGetLastError () != WSAEWOULDBLOCK)
			{	break;
			}
		}
					
		if (::WSAWaitForMultipleEvents (2, Handles, FALSE, 5000, FALSE) != (WSA_WAIT_EVENT_0 + 1))
		{	break;
		}

		if (::WSAEnumNetworkEvents (pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, &NetworkEvents) == SOCKET_ERROR)
		{	break;
		}

		if (!(NetworkEvents.lNetworkEvents & FD_CONNECT))
		{	break;
		}
 
		if (NetworkEvents.iErrorCode[FD_CONNECT_BIT] != 0)
		{	break;
		}

		if (::WSAEventSelect (pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, FD_CLOSE | FD_READ) == SOCKET_ERROR)
		{
		}
		RetCode = TRUE;
	}while (0);

	if (!RetCode)
	{
		DestroyDataConnection (pCfg);
		SendResponse (pCfg, "421 Failed to create data connection socket.\r\n");
		return FALSE;
	}


	return RetCode;
}

BOOL EFtpServer::DestroyDataConnection (EFtpClientCfg* pCfg)
{
	DBG_MSG(("EFtpServer::DestroyDataConnection - pCfg=%p, m_Data.m_hSocket=%p \r\n", pCfg, pCfg->m_Data.m_hSocket));

	if (pCfg->m_Data.m_hSocket != INVALID_SOCKET)
	{
		if (::WSAEventSelect (pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, 0) == SOCKET_ERROR)
		{
		}

		::closesocket (pCfg->m_Data.m_hSocket);
		pCfg->m_Data.m_hSocket = INVALID_SOCKET;
	}

	return TRUE;
}


