/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#pragma once

class EFtpServer
{
public:
	class EFtpClientCfg
	{
	public:
		EFtpClientCfg ()
		{
			m_pServer		= NULL;
			
			m_Main.m_hSocket	= INVALID_SOCKET;
			m_Main.m_hEvent		= ::WSACreateEvent ();
			
			m_Data.m_hSocket	= INVALID_SOCKET;
			m_Data.m_hEvent		= ::WSACreateEvent ();

			m_hThread		= NULL;
			m_ThreadId		= 0;
			m_nStatus		= EFtpServer::STATUS_LOGIN;
			m_LastError		= ERROR_SUCCESS;
			m_hStop			= NULL;

			m_bPassiveMode	= FALSE;
			m_RemotePort	= 0;
			m_RemoteAddress	= 0;

			m_pData			= NULL;
		}

		~EFtpClientCfg()
		{
			if (m_Main.m_hEvent != NULL)
			{
				::CloseHandle (m_Main.m_hEvent);
				m_Main.m_hEvent = NULL;
			}
			if (m_Data.m_hEvent != NULL)
			{
				::CloseHandle (m_Data.m_hEvent);
				m_Data.m_hEvent = NULL;
			}
			if (m_hThread != NULL)
			{
				::CloseHandle (m_hThread);
				m_hThread = NULL;
			}
		}

	public:
		EFtpServer*		m_pServer;

		struct
		{
			SOCKET			m_hSocket;
			HANDLE			m_hEvent;
		}m_Main;

		struct
		{
			SOCKET			m_hSocket;
			HANDLE			m_hEvent;
		}m_Data;

		HANDLE			m_hThread;
		ULONG			m_ThreadId;
		HANDLE			m_hStop;
		ULONG			m_LastError;
		BOOL			m_bPassiveMode;
		USHORT			m_RemotePort;
		ULONG			m_RemoteAddress;

		ULONG			m_nStatus;
		CHAR			m_szCurrentDir[MAX_PATH];
		CHAR			m_szUserName[128];
		SOCKADDR_IN		m_Peer;

		PVOID			m_pData;
	};

public:
	EFtpServer(void);
	~EFtpServer(void);

	BOOL			Start					();
	BOOL			Stop					();
	BOOL			IsRunning				();
	LONG			GetClientCount			();

	VOID			ClearUsers				();
	VOID			AddUser					(LPCSTR pUser, LPCSTR pPass);

	virtual VOID	OnConnect				(EFtpClientCfg* pCfg, SOCKADDR_IN* pAddr);
	virtual VOID	OnDisconnect			(EFtpClientCfg* pCfg, SOCKADDR_IN* pAddr);


protected:
	BOOL	Send							(EFtpClientCfg* pCfg, SOCKET hSocket, PVOID pBuffer, size_t SendLength, ULONG Timeout);
	BOOL	Receive							(EFtpClientCfg* pCfg, SOCKET hSocket, HANDLE hEvent, PVOID pBuffer, size_t BufferLength, PULONG pBytesReceive, ULONG Timeout);
	BOOL	CreateDataConnection			(EFtpClientCfg* pCfg);
	BOOL	CreatePasvDataConnection		(EFtpClientCfg* pCfg);
	BOOL	DestroyDataConnection			(EFtpClientCfg* pCfg);

	virtual	VOID	ParseCommandLine		(EFtpClientCfg* pCfg, PUCHAR pBuffer);
	virtual	BOOL	SendResponse			(EFtpClientCfg* pCfg, LPCSTR pstrFormat, ...);
	virtual	VOID	ProcessCommand			(EFtpClientCfg* pCfg, LPCSTR Cmd, LPCSTR Arg);
	virtual	VOID	SendSyntaxError			(EFtpClientCfg* pCfg);

	virtual	VOID	ProcessABOR				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessCWD				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessSITE				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessDELE				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessHELP				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessLIST				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessMKD				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessNOOP				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessPASS				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessPASV				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessPORT				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessPWD				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessQUIT				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessREST				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessRETR				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessRMD				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessRNFR				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessRNTO				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessSIZE				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessSTOR				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessSYST				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessTYPE				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);
	virtual	VOID	ProcessUSER				(EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments);


protected:
static	ULONG WINAPI	FtpListenThread (LPVOID pParam);
		VOID			FtpListenThread ();
static	ULONG WINAPI	FtpClientThread (LPVOID pParam);
		VOID			FtpClientThread (EFtpClientCfg* pCfg);


protected:
	HANDLE			m_hThread;
	HANDLE			m_hStop;
	LONG			m_ClientRunning;
	WSAOVERLAPPED	m_WsaOverlap;

	EMapStringToStringA	m_UserMap;

	enum
	{
		STATUS_IDLE,
		STATUS_LOGIN,
		STATUS_LOGIN_USER,
		STATUS_LIST,
		STATUS_UPLOAD,
		STATUS_DOWNLOAD,
	};

};
