/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#pragma once

#pragma warning(disable:4995)
#include <list>
#pragma warning(default:4995)


template<class TYPE, class ARG_TYPE = const TYPE&>
class EList
{
public:
	EList ();
	virtual ~EList ();

	VOID		Lock			();
	VOID		Unlock			();

	BOOL		IsEmpty			() const;
	INT_PTR		GetCount		() const;

	TYPE		RemoveHead		();
	TYPE		RemoveTail		();
	VOID		RemoveAll		();

	VOID		AddHead			(ARG_TYPE Data);
	VOID		AddTail			(ARG_TYPE Data);

	HANDLE		GetHeadHandle	() const;
	HANDLE		GetTailHandle	() const;
	TYPE&		GetNext			(HANDLE& hList);
	const TYPE&	GetNext			(HANDLE& hList) const;

	TYPE&		GetAt			(HANDLE hList);
	const TYPE&	GetAt			(HANDLE hList) const;
	void		SetAt			(HANDLE hList, ARG_TYPE Data);

	void		RemoveAt		(HANDLE hList);

protected:
	std::list <TYPE>	m_List;
	CRITICAL_SECTION	m_Lock;
};

template<class TYPE, class ARG_TYPE>
inline EList<TYPE, ARG_TYPE>::EList ()
{
	::InitializeCriticalSection (&m_Lock);
}

template<class TYPE, class ARG_TYPE>
inline EList<TYPE, ARG_TYPE>::~EList ()
{
	RemoveAll();
	::DeleteCriticalSection (&m_Lock);
}

template<class TYPE, class ARG_TYPE>
inline VOID EList<TYPE, ARG_TYPE>::Lock ()
{
	::EnterCriticalSection (&m_Lock);
}

template<class TYPE, class ARG_TYPE>
inline VOID EList<TYPE, ARG_TYPE>::Unlock ()
{
	::LeaveCriticalSection (&m_Lock);
}

template<class TYPE, class ARG_TYPE>
inline BOOL EList<TYPE, ARG_TYPE>::IsEmpty () const
{
	return m_List.empty ();
}

template<class TYPE, class ARG_TYPE>
inline INT_PTR EList<TYPE, ARG_TYPE>::GetCount () const
{
	return m_List.size ();
}

template<class TYPE, class ARG_TYPE>
inline TYPE EList<TYPE, ARG_TYPE>::RemoveHead ()
{
	std::list <TYPE>::iterator Iter;

	Iter = m_List.begin ();
	TYPE Data = *Iter;
	m_List.pop_front ();
	return Data;
}

template<class TYPE, class ARG_TYPE>
inline TYPE EList<TYPE, ARG_TYPE>::RemoveTail ()
{
	std::list <TYPE>::iterator Iter;

	Iter = m_List.rbegin ();
	TYPE Data = *Iter;
	m_List.pop_back ();
	return Data;
}

template<class TYPE, class ARG_TYPE>
inline void EList<TYPE, ARG_TYPE>::RemoveAll ()
{
	m_List.clear ();
}

template<class TYPE, class ARG_TYPE>
inline VOID EList<TYPE, ARG_TYPE>::AddHead (ARG_TYPE Data)
{
	m_List.push_front (Data);
}

template<class TYPE, class ARG_TYPE>
inline VOID EList<TYPE, ARG_TYPE>::AddTail (ARG_TYPE Data)
{
	m_List.push_back (Data);
}

template<class TYPE, class ARG_TYPE>
inline HANDLE EList<TYPE, ARG_TYPE>::GetHeadHandle () const
{	
	if (IsEmpty ())
	{	return NULL;
	}
	return (HANDLE)1; 
}

template<class TYPE, class ARG_TYPE>
inline HANDLE EList<TYPE, ARG_TYPE>::GetTailHandle () const
{
	return (HANDLE)m_List.size ();
}

template<class TYPE, class ARG_TYPE>
inline TYPE& EList<TYPE, ARG_TYPE>::GetNext (HANDLE& hList)
{
	std::list <TYPE>::iterator Iter = m_List.begin ();
	INT	Index = (INT)hList;

	for (int i=1;i<Index;i++)
	{
		Iter++;
	}
	Index++;

	if (GetCount () < Index)
	{
		hList = NULL;
	}
	else
	{
		hList = (HANDLE)Index;
	}

	return *Iter;
}

template<class TYPE, class ARG_TYPE>
inline const TYPE& EList<TYPE, ARG_TYPE>::GetNext (HANDLE& hList) const
{ 
	std::list <TYPE>::const_iterator Iter = m_List.begin ();
	INT	Index = (INT)hList;

	for (int i=1;i<Index;i++)
	{
		Iter++;
	}
	Index++;

	if (GetCount () < Index)
	{
		hList = NULL;
	}
	else
	{
		hList = (HANDLE)Index;
	}

	return *Iter;
}

template<class TYPE, class ARG_TYPE>
inline TYPE& EList<TYPE, ARG_TYPE>::GetAt (HANDLE hList)
{
	std::list <TYPE>::iterator Iter = m_List.begin ();
	INT	Index = (INT)hList;

	for (int i=1;i<Index;i++)
	{
		Iter++;
	}

	return *Iter;
}

template<class TYPE, class ARG_TYPE>
inline const TYPE& EList<TYPE, ARG_TYPE>::GetAt (HANDLE hList) const
{
	std::list <TYPE>::const_iterator Iter = m_List.begin ();
	INT	Index = (INT)hList;

	for (int i=1;i<Index;i++)
	{
		Iter++;
	}

	return *Iter;
}

template<class TYPE, class ARG_TYPE>
inline void EList<TYPE, ARG_TYPE>::SetAt (HANDLE hList, ARG_TYPE Data)
{
	std::list <TYPE>::iterator Iter = m_List.begin ();
	INT	Index = (INT)hList;

	for (int i=1;i<Index;i++)
	{
		Iter++;
	}

	*Iter = Data;
}


template<class TYPE, class ARG_TYPE>
void EList<TYPE, ARG_TYPE>::RemoveAt (HANDLE hList)
{
	std::list <TYPE>::const_iterator Iter = m_List.begin ();
	INT	Index = (INT)hList;

	for (int i=1;i<Index;i++)
	{
		Iter++;
	}
	m_List.erase (Iter);
}
