/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "StdAfx.h"
#include "EListCtrl.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif



EListCtrl::EListCtrl(void)
{
	m_SelectedColumn = -2;
}

EListCtrl::~EListCtrl(void)
{
}

BOOL EListCtrl::Create (ULONG ExStyle, ULONG Style, int x, int y, int Width, int Height, HWND hWndParent, UINT_PTR Id)
{
	INITCOMMONCONTROLSEX init;
	init.dwSize = sizeof(init);
	init.dwICC = ICC_LISTVIEW_CLASSES;
	
	if (!::InitCommonControlsEx (&init))
	{	return FALSE;
	}

	return CreateEx (ExStyle | LVS_EX_FULLROWSELECT, WC_LISTVIEW, NULL, Style, x, y, Width, Height, hWndParent, (HMENU)Id);
}

BOOL EListCtrl::CreateEx (DWORD dwExStyle, LPCTSTR lpClassName, LPCTSTR lpWindowName, DWORD dwStyle, int x, int y, int nWidth, int nHeight, HWND hWndParent, HMENU hMenu)
{
	dwStyle |= LVS_REPORT | LVS_OWNERDRAWFIXED;

	return EWindow::CreateEx (dwExStyle, lpClassName, lpWindowName, dwStyle, x, y, nWidth, nHeight, hWndParent, hMenu);
}


BOOL EListCtrl::OnCreate (CREATESTRUCT* pStruct)
{
	return TRUE;
}

VOID EListCtrl::OnDestroy ()
{
}

int EListCtrl::InsertColumn(int Col, LPCTSTR pszColumnText, int Format, int Width)
{
	LVCOLUMN Column;

	Column.mask		= LVCF_TEXT | LVCF_FMT | LVCF_WIDTH;
	Column.pszText	= (LPTSTR)pszColumnText;
	Column.fmt		= Format;
	Column.cx		= Width;
	
	if (Width == -1)
	{
		Column.mask &= ~LVCF_WIDTH;
	}

	return ListView_InsertColumn (m_hWnd, Col, &Column);
}

int EListCtrl::InsertColumn (int Col, const LVCOLUMN* pLvColumn)
{
	ASSERT(::IsWindow(m_hWnd));

	return ListView_InsertColumn (m_hWnd, Col, pLvColumn);
}

BOOL EListCtrl::DeleteColumn(int Col)
{
	ASSERT(::IsWindow(m_hWnd));

	return ListView_DeleteColumn (m_hWnd, Col);
}

int EListCtrl::InsertItem (UINT Mask, int Item, LPCTSTR pszItem, UINT State, UINT StateMask, int Image, LPARAM lParam)
{
	LVITEM LvItem;

	LvItem.mask			= Mask;
	LvItem.iItem		= Item;
	LvItem.iSubItem		= 0;
	LvItem.pszText		= (LPTSTR)pszItem;
	LvItem.state		= State;
	LvItem.stateMask	= StateMask;
	LvItem.iImage		= Image;
	LvItem.lParam		= lParam;

	return ListView_InsertItem (m_hWnd, &LvItem);

}

int EListCtrl::InsertItem(LVITEM* pLvItem)
{
	ASSERT(::IsWindow(m_hWnd));

	return ListView_InsertItem (m_hWnd, pLvItem);
}



BOOL EListCtrl::DeleteItem (int Item)
{
	ASSERT(::IsWindow(m_hWnd));
	
	return ListView_DeleteItem (m_hWnd, Item);
}

BOOL EListCtrl::DeleteAllItems ()
{ 
	ASSERT(::IsWindow(m_hWnd));
	
	return ListView_DeleteAllItems (m_hWnd);
}

int EListCtrl::GetItemCount() const
{
	ASSERT(::IsWindow(m_hWnd));
	
	return ListView_GetItemCount (m_hWnd);
}

BOOL EListCtrl::GetItem(LVITEM* pLvItem) const
{
	ASSERT(::IsWindow(m_hWnd));
	
	return ListView_GetItem (m_hWnd, pLvItem);
}

BOOL EListCtrl::SetItem(const LVITEM* pLvItem)
{
	ASSERT(::IsWindow(m_hWnd));
	
	return ListView_SetItem (m_hWnd, pLvItem);
}

VOID EListCtrl::SetItemText (INT Item, INT Column, LPCTSTR pszText)
{
	ASSERT(::IsWindow(m_hWnd));
	
	ListView_SetItemText (m_hWnd, Item, Column, (LPTSTR)pszText);
}

VOID EListCtrl::GetItemText(INT Item, INT Column, LPTSTR pszText, INT cchTextMax)
{
	ListView_GetItemText (m_hWnd, Item, Column, pszText, cchTextMax);
}

BOOL EListCtrl::EnsureVisible(int Item, BOOL bPartialOK)
{
	ASSERT(::IsWindow(m_hWnd));

	return ListView_EnsureVisible (m_hWnd, Item, bPartialOK);
}

BOOL EListCtrl::RedrawItems(int First, int Last)
{
	ASSERT(::IsWindow(m_hWnd));

	return ListView_RedrawItems (m_hWnd, First, Last);
}

BOOL EListCtrl::OnDrawItemRefrection (WPARAM CtrlId, LPDRAWITEMSTRUCT pDrawItemStruct)
{
    HDC			hDC				= NULL;
	RECT		rcBounds		= {0};
	RECT		rcLabel			= {0};
	RECT		rcSubItem		= {0};
    HBRUSH		hBrush			= NULL;
	COLORREF	BkColor			= 0;
	COLORREF	TextColor		= 0;
	LPTSTR		pszText			= NULL;
	LVCOLUMN	LvColumn		= {0};
	LVITEM		LvItem			= {0};
	HIMAGELIST	hImage			= ListView_GetImageList (m_hWnd, LVSIL_SMALL);
	HWND		hHeaderCtrl		= ListView_GetHeader(m_hWnd);
	int			HdrItemCount	= 0;
	TCHAR		szText[512];

	do
	{
		if (hHeaderCtrl == NULL)
		{	break;
		}

		hDC = pDrawItemStruct->hDC;
		if (hDC == NULL)
		{	break;
		}

		pszText = szText;//new TCHAR[LISTCTRL_MAX_TEXT_SIZE];

		::SaveDC (hDC);
		::SetBkMode(hDC, OPAQUE);

		ListView_GetItemRect (m_hWnd, pDrawItemStruct->itemID, &rcBounds, LVIR_BOUNDS);

		BkColor		= ListView_GetTextBkColor (m_hWnd);
		TextColor	= ListView_GetTextColor (m_hWnd);

		if (TextColor == CLR_DEFAULT)
		{	TextColor = ::GetSysColor (COLOR_WINDOWTEXT);
		}

		if (BkColor == CLR_DEFAULT)
		{	BkColor = ::GetSysColor (COLOR_WINDOW);
		}
		
		hBrush = ::CreateSolidBrush (BkColor);
		if (hBrush == NULL)
		{	break;
		}
		
		::FillRect (hDC, &rcBounds, hBrush);
		::DeleteObject (hBrush);
		hBrush = NULL;

		HdrItemCount = Header_GetItemCount (hHeaderCtrl);
		for (int Column = 0;Column < HdrItemCount;Column++)
		{
			LvItem.iItem		= pDrawItemStruct->itemID;
			LvItem.iSubItem		= Column;
			LvItem.mask			= LVIF_IMAGE | LVIF_TEXT;
			LvItem.pszText		= pszText;
			LvItem.cchTextMax	= LISTCTRL_MAX_TEXT_SIZE;
			ListView_GetItem (m_hWnd, &LvItem);
			
			LvColumn.mask		= LVCF_FMT | LVCF_WIDTH;
			ListView_GetColumn (m_hWnd, Column, &LvColumn);

			ListView_GetSubItemRect (m_hWnd, pDrawItemStruct->itemID, Column, LVIR_LABEL, &rcSubItem);

			if ((pDrawItemStruct->itemState & ODS_SELECTED) && ((m_SelectedColumn == Column) || (m_SelectedColumn == -2)))
			{
				::SetBkColor (hDC, ::GetSysColor(COLOR_HIGHLIGHT));
				::SetTextColor (hDC, ::GetSysColor(COLOR_HIGHLIGHTTEXT));
				hBrush = ::CreateSolidBrush (::GetSysColor(COLOR_HIGHLIGHT));
				if (hBrush != NULL)
				{	
					::FillRect (hDC, &rcSubItem, hBrush);
					::DeleteObject (hBrush);
					hBrush = NULL;
				}
			}
			else
			{
				::SetBkColor (hDC, BkColor);
				::SetTextColor (hDC, TextColor);
			}


			if (hImage)
			{
				IMAGEINFO ImageInfo = {0};
				
				if (ImageList_GetImageInfo (hImage, LvItem.iImage, &ImageInfo))
				{
					ImageList_Draw (hImage, LvItem.iImage, hDC, rcSubItem.left, rcSubItem.top, ((pDrawItemStruct->itemState & ODS_SELECTED)? ILD_FOCUS:ILD_NORMAL) |ILD_TRANSPARENT);
					rcSubItem.left += ImageInfo.rcImage.right - ImageInfo.rcImage.left;
				}
			}
			
			UINT Format = 0;
			
			switch (LvColumn.fmt & LVCFMT_JUSTIFYMASK)
			{
			case LVCFMT_RIGHT:	{	Format = DT_RIGHT;	}	break;
			case LVCFMT_CENTER:	{	Format = DT_CENTER;	}	break;
			default:			{	Format = DT_LEFT;	}	break;
			}

			rcSubItem.left += 2;
			::DrawText (hDC, pszText, -1, &rcSubItem, Format | DT_SINGLELINE | DT_NOPREFIX | DT_END_ELLIPSIS | DT_VCENTER);

		}

		if (pDrawItemStruct->itemState & ODS_FOCUS)
		{
			::DrawFocusRect (hDC, &rcBounds);
		}
	}
	while (0);

	if (hDC != NULL)
	{	::RestoreDC (hDC, -1);
	}
	
	if (hBrush != NULL)
	{
		::DeleteObject (hBrush);
	}

	//if (pszText != NULL)
	//{
	//	delete pszText;
	//}

	return TRUE;
}



