/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#pragma once

#include <map>



//-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-
///-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/
//-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-
///-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/

template<class KEY, class VALUE>
class EMap
{
public:
	EMap ();
	virtual ~EMap ();

	VOID			Lock			();
	VOID			Unlock			();

	INT_PTR			GetCount		() const;
	BOOL			IsEmpty			() const;

	void			SetAt			(KEY key, VALUE newValue);
	BOOL			Lookup			(KEY key, VALUE& rValue);

	BOOL			RemoveKey		(KEY key);
	void			RemoveAll		();

	HANDLE			GetStartHandle	() const;
	void			GetNext			(HANDLE& hMap, KEY& rKey, VALUE& rValue) const;

protected:
	std::map <KEY, VALUE> m_Map;
	CRITICAL_SECTION	m_Lock;
};


template<class KEY, class VALUE>
EMap<KEY, VALUE>::EMap ()
{
	::InitializeCriticalSection (&m_Lock);
}

template<class KEY, class VALUE>
EMap<KEY, VALUE>::~EMap ()
{
	RemoveAll ();
	::DeleteCriticalSection (&m_Lock);
}

template<class KEY, class VALUE>
inline VOID EMap<KEY, VALUE>::Lock ()
{
	::EnterCriticalSection (&m_Lock);
}

template<class KEY, class VALUE>
inline VOID EMap<KEY, VALUE>::Unlock ()
{
	::LeaveCriticalSection (&m_Lock);
}


template<class KEY, class VALUE>
inline INT_PTR EMap<KEY, VALUE>::GetCount () const
{
	return m_Map.size ();
}

template<class KEY, class VALUE>
inline BOOL EMap<KEY, VALUE>::IsEmpty () const
{
	return m_Map.empty ();
}

template<class KEY, class VALUE>
inline void EMap<KEY, VALUE>::SetAt (KEY Key, VALUE Value)
{
	m_Map[Key] = Value;
}





template<class KEY, class VALUE>
BOOL EMap<KEY, VALUE>::Lookup (KEY Key, VALUE& rValue)
{
	std::map <KEY, ARG_KEY>::const_iterator Iter = m_Map.find (Key);

	if (Iter == m_Map.end ())
	{	return FALSE;
	}
	rValue = Iter->second;
	return TRUE;
}

template<class KEY, class VALUE>
BOOL EMap<KEY, VALUE>::RemoveKey (KEY Key)
{
	std::map <KEY, ARG_KEY>::const_iterator Iter = m_Map.find (Key);

	if (Iter == m_Map.end ())
	{	return FALSE;
	}

	m_Map.erase (Iter);

	return TRUE;
}


template<class KEY, class VALUE>
void EMap<KEY, VALUE>::RemoveAll ()
{
	m_Map.clear ();
}


template<class KEY, class VALUE>
inline HANDLE EMap<KEY, VALUE>::GetStartHandle () const
{
	if (IsEmpty ())
	{	return NULL;
	}
	
	return (HANDLE)1;
}


template<class KEY, class VALUE>
void EMap<KEY, VALUE>::GetNext (HANDLE& hMap, KEY& rKey, VALUE& rValue) const
{
	std::map <KEY, ARG_KEY>::const_iterator Iter = m_Map.begin ();
	INT Index = (INT)hMap;

	for (int i=1;i<Index;i++)
	{
		Iter++;
	}
	Index++;

	if (GetCount () < Index)
	{	hMap = NULL;
	}
	else
	{	hMap = (HANDLE)Index;
	}
	rKey = Iter->first;
	rValue = Iter->second;
}



















//-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-
///-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/
//-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-
///-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/-/
class EMapStringToStringW : public EMap<ULONGLONG, std::wstring>
{
public:
	inline ULONGLONG CalcHash (LPCWSTR key)
	{
		ULONGLONG nHash	= 0;
		while (*key)
		{	
			nHash += *key++;
			nHash += (nHash<<5);
		}
		return nHash;
	}

	inline void SetAt (LPCWSTR key, LPCWSTR Value)
	{
		m_Map[CalcHash(key)] = Value;
		m_KeyMap[CalcHash(key)] = key;
	}

	inline BOOL RemoveKey (LPCWSTR key)
	{
		std::map <ULONGLONG, std::wstring>::const_iterator Iter = m_Map.find (CalcHash(key));
		std::map <ULONGLONG, std::wstring>::const_iterator KeyIter = m_KeyMap.find (CalcHash(key));
		BOOL	RetCode = FALSE;

		if (Iter != m_Map.end ())
		{	
			RetCode = TRUE;
			m_Map.erase (Iter);
		}

		if (KeyIter != m_KeyMap.end ())
		{	m_KeyMap.erase (KeyIter);
		}

		return RetCode;
	}

	inline BOOL	Lookup (LPCWSTR key, LPCWSTR& rValue)
	{
		std::map <ULONGLONG, std::wstring>::const_iterator Iter = m_Map.find (CalcHash(key));

		if (Iter == m_Map.end ())
		{	return FALSE;
		}

		rValue = Iter->second.c_str ();
		return TRUE;
	}

	inline void	GetNext	(HANDLE& hMap, LPCWSTR& rKey, LPCWSTR& rValue) const
	{
		std::map <ULONGLONG, std::wstring>::const_iterator Iter = m_Map.begin ();
		std::map <ULONGLONG, std::wstring>::const_iterator KeyIter;
		INT Index = (INT)hMap;

		for (int i=1;i<Index;i++)
		{
			Iter++;
		}
		Index++;

		if (GetCount () < Index)
		{	hMap = NULL;
		}
		else
		{	hMap = (HANDLE)Index;
		}
		KeyIter = m_KeyMap.find (Iter->first);
		rKey = KeyIter->second.c_str ();
		rValue = Iter->second.c_str ();
	}


public:
	std::map <ULONGLONG, std::wstring> m_KeyMap;
};

///////////////////////////////////////////////////////////////////////////////////////////////////
class EMapStringToStringA : public EMap<ULONGLONG, std::string>
{
public:
	inline ULONGLONG CalcHash (LPCSTR key)
	{
		ULONGLONG nHash	= 0;
		while (*key)
		{	
			nHash += *key++;
			nHash += (nHash<<5);
		}
		return nHash;
	}

	inline void SetAt (LPCSTR key, LPCSTR Value)
	{
		m_Map[CalcHash(key)] = Value;
		m_KeyMap[CalcHash(key)] = key;
	}

	inline BOOL RemoveKey (LPCSTR key)
	{
		std::map <ULONGLONG, std::string>::const_iterator Iter = m_Map.find (CalcHash(key));
		std::map <ULONGLONG, std::string>::const_iterator KeyIter = m_KeyMap.find (CalcHash(key));
		BOOL	RetCode = FALSE;

		if (Iter != m_Map.end ())
		{	
			RetCode = TRUE;
			m_Map.erase (Iter);
		}

		if (KeyIter != m_KeyMap.end ())
		{	m_KeyMap.erase (KeyIter);
		}

		return RetCode;
	}

	inline BOOL	Lookup (LPCSTR key, LPCSTR& rValue)
	{
		std::map <ULONGLONG, std::string>::const_iterator Iter = m_Map.find (CalcHash(key));

		if (Iter == m_Map.end ())
		{	return FALSE;
		}

		rValue = Iter->second.c_str ();
		return TRUE;
	}

	inline void	GetNext	(HANDLE& hMap, LPCSTR& rKey, LPCSTR& rValue) const
	{
		std::map <ULONGLONG, std::string>::const_iterator Iter = m_Map.begin ();
		std::map <ULONGLONG, std::string>::const_iterator KeyIter;
		INT Index = (INT)hMap;

		for (int i=1;i<Index;i++)
		{
			Iter++;
		}
		Index++;

		if (GetCount () < Index)
		{	hMap = NULL;
		}
		else
		{	hMap = (HANDLE)Index;
		}
		KeyIter = m_KeyMap.find (Iter->first);
		rKey = KeyIter->second.c_str ();
		rValue = Iter->second.c_str ();
	}


public:
	std::map <ULONGLONG, std::string> m_KeyMap;
};




#ifdef _UNICODE
#define EMapStringToString EMapStringToStringW
#else
#define EMapStringToString EMapStringToStringA
#endif
