/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "StdAfx.h"
#include "ERdFtpServer.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif


#define SEND_TIME_OUT		(20 * 1000)
#define RECEIVE_TIME_OUT	(20 * 1000)

ERdFtpServer::ERdFtpServer(void)
{
	size_t Length = 0;

	::ZeroMemory (m_szSaveFolder, sizeof (m_szSaveFolder));
	::GetModuleFileName (NULL, m_szSaveFolder, sizeof (m_szSaveFolder));

	if (SUCCEEDED (StringCchLength (m_szSaveFolder, _countof (m_szSaveFolder), &Length)))
	{	
		while (Length)
		{
			Length--;
			if (m_szSaveFolder[Length] == '\\')
			{	break;
			}
			m_szSaveFolder[Length] = 0;
		}
	}
	m_bAddTimeDate = TRUE;
	m_bMakeInfoTxt = FALSE;

	m_pCallback			= NULL;
	m_pCallbackContext	= NULL;

}

ERdFtpServer::~ERdFtpServer(void)
{
	int x=0;
}

VOID ERdFtpServer::SetSaveFolder (LPCTSTR pszFolder)
{
	size_t Length = 0;

	StringCchCopy (m_szSaveFolder, _countof (m_szSaveFolder), pszFolder);

	if (SUCCEEDED (StringCchLength (m_szSaveFolder, _countof (m_szSaveFolder), &Length)))
	{	
		if (m_szSaveFolder[Length - 1] != '\\')
		{
			StringCchCat (m_szSaveFolder, _countof (m_szSaveFolder), _T("\\"));
		}
	}
}

LPCTSTR ERdFtpServer::GetSaveFolder ()
{
	return m_szSaveFolder;
}

VOID ERdFtpServer::CreateDirectory (LPCTSTR pszFolder)
{
	LPCTSTR	pszTmp;
	TCHAR	szFolder[MAX_PATH] = {0};

	if (pszFolder == NULL)
	{	return;
	}

	pszTmp = StringCchStrStr (pszFolder, MAX_PATH, _T("\\\\"));//network
	
	if (pszTmp != NULL)//local drive
	{
		pszTmp += 2;
	}
	else
	{	pszTmp = pszFolder;
	}

	pszTmp = StringCchStrStr (pszTmp, MAX_PATH, _T("\\"));//network name or drive
	if (pszTmp == NULL)
	{	return;
	}

	pszTmp = StringCchStrStr (pszTmp+1, MAX_PATH, _T("\\"));//folder
	if (pszTmp == NULL)
	{	return;
	}

	while (pszTmp)
	{
		StringCchCopyN (szFolder, _countof (szFolder), pszFolder, pszTmp - m_szSaveFolder);
		::CreateDirectory (szFolder, NULL);
		pszTmp = StringCchStrStr (pszTmp+1, MAX_PATH, _T("\\"));
	}
}

VOID ERdFtpServer::SetCallBack (ERDFTPSERVER_CALLBACK pCallback, PVOID pContext)
{
	m_pCallback = pCallback;
	m_pCallbackContext = pContext;
}

VOID ERdFtpServer::Callback (ULONG Type, ERdData* pData)
{
	switch (Type)
	{
	case TYPE_CONNECT:
	case TYPE_DISCONNECT:
	case TYPE_FILE_CREATE_SUCCESS:
	case TYPE_STATUS_ERROR:
	case TYPE_STATUS_UPDATE:
	case TYPE_RECEIVE_COMPLETE:
		{
		}
		break;

	}

	if (m_pCallback != NULL)
	{
		m_pCallback (Type, pData, m_pCallbackContext);
	}
}




VOID ERdFtpServer::OnConnect (EFtpClientCfg* pCfg, SOCKADDR_IN* pAddr)
{
	ERdData* pData = new ERdData;

	if (pData == NULL)
	{
		SendResponse (pCfg, "421 Failed to create data.\r\n");
		return;
	}

	pData->m_SocketAddress = *pAddr;
	pCfg->m_pData = pData;

	SendResponse (pCfg, "220 Connected. <00:00:00:000>\r\n");

	Callback (TYPE_CONNECT, pData);
}

VOID ERdFtpServer::OnDisconnect (EFtpClientCfg* pCfg, SOCKADDR_IN* pAddr)
{
	ERdData* pData = (ERdData*)pCfg->m_pData;

	if (pData != NULL)
	{
		Callback (TYPE_DISCONNECT, pData);

		pCfg->m_pData = NULL;
		delete pData;
	}
}

VOID ERdFtpServer::ProcessRETR (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("ERdFtpServer::ProcessRETR - pCfg=%p, %s, %s \r\n", pCfg, pszCommand, pszArguments));

	if (lstrcmpiA (pszArguments, "$xml$status.xml") == 0)
	{	
		StatusXmlRETR (pCfg);
		return;
	}

	EFtpServer::ProcessRETR (pCfg, pszCommand, pszArguments);
}

VOID ERdFtpServer::ProcessSTOR (EFtpClientCfg* pCfg, LPCSTR pszCommand, LPCSTR pszArguments)
{
	DBG_MSG(("ERdFtpServer::ProcessSTOR - pCfg=%p, %s, %s \r\n", pCfg, pszCommand, pszArguments));

	if (lstrcmpiA (pszArguments, "$netdubbing$dubbinginfo.xml") == 0)
	{	
		DubbingInfoXmlSTOR (pCfg);
		return;
	}

	if (lstrcmpiA (pszArguments, "$netdubbing$command.xml") == 0)
	{	
		DubbingCommandXmlSTOR (pCfg);
		return;
	}

	if (lstrcmpiA (pszArguments, "$netdubbing$dev0.dat") == 0)
	{	
		DubbingDatSTOR (pCfg);
		return;
	}

	EFtpServer::ProcessSTOR (pCfg, pszCommand, pszArguments);
}

VOID ERdFtpServer::DubbingInfoXmlSTOR (EFtpClientCfg* pCfg)
{
	UCHAR		pBuffer[1024 * 16]	= {0};
	//CHAR		szXml[1024 * 16]	= {0};
	ULONG		RecvLength;

	if (!CreateDataConnection (pCfg))
	{
		return;
	}

	SendResponse (pCfg, "150 Opening BINARY mode data connection for file transfer.\r\n");

	do
	{
		while (Receive (pCfg, pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, pBuffer, sizeof (pBuffer), &RecvLength, RECEIVE_TIME_OUT))
		{
			//pBuffer[RecvLength] = 0;
			//StringCchCatA (szXml, _countof (szXml), (PCHAR)pBuffer);
		}
	}
	while (0);

	DestroyDataConnection (pCfg);

	SendResponse (pCfg, "226 Transfer complete.\r\n");

	//DBG_MSG(("%s\n", szXml));
}

VOID ERdFtpServer::StatusXmlRETR (EFtpClientCfg* pCfg)
{
	CHAR			szStatusXml[1024 * 2]	= {0};
	ULARGE_INTEGER	FreeBytesAvailable		= {0};
	ULARGE_INTEGER	TotalNumberOfBytes		= {0};
	ULARGE_INTEGER	TotalNumberOfFreeBytes	= {0};
	size_t			Length					= 0;
	
	CreateDirectory (m_szSaveFolder);

	if (!::GetDiskFreeSpaceEx (m_szSaveFolder, &FreeBytesAvailable, &TotalNumberOfBytes, &TotalNumberOfFreeBytes))
	{
	}

	StringCchPrintfA (szStatusXml, _countof (szStatusXml), "<?xml version=\"1.0\" encoding=\"shift_jis\"?>\r\n"
							"<status>\r\n"
							"<dubbing>ready</dubbing>\r\n"
							"<accept_dubbing>ready</accept_dubbing>\r\n"
							"<device0_remain>%I64u</device0_remain>\r\n"
							"<device0_recorded>%I64u</device0_recorded>\r\n"
							"<device0_title_remain>350</device0_title_remain>\r\n"
							"</status>\r\n"
							, FreeBytesAvailable.QuadPart / 2048
							, (TotalNumberOfBytes.QuadPart - FreeBytesAvailable.QuadPart) / 2048);


	if (!CreateDataConnection (pCfg))
	{
		return;
	}

	SendResponse (pCfg, "150 Opening BINARY mode data connection for file transfer.\r\n");

	if (FAILED (StringCchLengthA (szStatusXml, _countof (szStatusXml), &Length)))
	{	
		SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");
		DestroyDataConnection (pCfg);
		return;
	}

	if (!Send (pCfg, pCfg->m_Data.m_hSocket, szStatusXml, Length, SEND_TIME_OUT))
	{	
		SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");
		DestroyDataConnection (pCfg);
		return;
	}
				
	DestroyDataConnection (pCfg);
	SendResponse (pCfg, "226 Transfer complete\r\n");
}

VOID ERdFtpServer::DubbingDatSTOR (EFtpClientCfg* pCfg)
{
	PUCHAR		pBuffer					= NULL;
	ULONG		BufferLength			= 64 * 1024 - 1;
	ULONG		RecvLength				= 0;
	UCHAR		pHeader[1024 * 4]		= {0};
	ULONG		HeaderLength			= 0;
	CHAR		szTitleInfo[0x320]		= {0};
	CHAR		szTitleChannel[0x40]	= {0};
	CHAR		szTitleName[65]			= {0};
	TCHAR		szTitleNameFile[256]	= {0};
	TCHAR		szTemp[2048]			= {0};
	ERdData*	pData					= (ERdData*)pCfg->m_pData;
	INT			FileIndex				= 0;
	HANDLE		hFile					= INVALID_HANDLE_VALUE;
	ULONG		BytesWritten			= 0;
	BOOL		bLoop					= TRUE;

	CreateDirectory (m_szSaveFolder);

	do
	{
		if (pData == NULL)
		{	break;
		}

		if (!CreateDataConnection (pCfg))
		{
			break;
		}

		pBuffer = new UCHAR[BufferLength];
		if (pBuffer == NULL)
		{
			SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");
			break;
		}

		SendResponse (pCfg, "150 Opening BINARY mode data connection for file transfer.\r\n");

		pData->m_StartTime = ::GetTickCount ();
		pData->m_sTime= ::GetTickCount ();

		//wb_[2048oCg炢M
		while (Receive (pCfg, pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, pBuffer, 1024, &RecvLength, RECEIVE_TIME_OUT))
		{
			memcpy (pHeader + HeaderLength, pBuffer, RecvLength);
			HeaderLength += RecvLength;
			if (HeaderLength >= 2048)
			{	break;
			}
		}

		//wb_[ǂݍ߂Ȃ̂ŏI
		if (HeaderLength < 2048)
		{
			DestroyDataConnection (pCfg);
			SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");
			break;
		}

		memcpy (szTitleName, &pHeader[0x84], 0x40);
		memcpy (szTitleInfo, &pHeader[0xCC], 0x31f);
		memcpy (szTitleChannel, &pHeader[0x434], 0x3f);

		for (int i=0;i<64;i++)
		{	
			switch (pHeader[0x84 + i])
			{
			case '\\':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T(""));	}	break;
			case '/':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T("^"));	}	break;
			case ':':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T("F"));	}	break;
			case '*':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T(""));	}	break;
			case '?':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T("H"));	}	break;
			case '\"':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T("h"));	}	break;
			case '<':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T(""));	}	break;
			case '>':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T(""));	}	break;
			case '|':	{	StringCchCat (szTitleNameFile, _countof (szTitleNameFile), _T("b"));	}	break;
			case 0x00:	{	i = 64;								}	break;
			default:	
				{
					TCHAR Char[2] = {(TCHAR)pHeader[0x84 + i], 0};
					StringCchCat (szTitleNameFile, _countof (szTitleNameFile), Char);
				}
				break;
			}
		}

		//t@C̒擾
		pData->m_FileLength = MAKELONG (MAKEWORD (pHeader[0x42], pHeader[0x41]), MAKEWORD (pHeader[0x40], pHeader[0x3F]));
		pData->m_FileLength *= 2048;

		//t@Cɓǉ
		if (m_bAddTimeDate)
		{
#ifdef _UNICODE
			StringCchPrintf (szTemp, _countof (szTemp), _T("_%C%C%C%C%C%C_%C%C%C%C")
					, pHeader[0x62], pHeader[0x63] //N 
					, pHeader[0x64], pHeader[0x65] //
					, pHeader[0x66], pHeader[0x67] //
					, pHeader[0x6c], pHeader[0x6d] //Jn
					, pHeader[0x6e], pHeader[0x6f] //Jn
					);
#else
			StringCchPrintf (szTemp, _countof (szTemp), _T("_%c%c%c%c%c%c_%c%c%c%c")
					, pHeader[0x62], pHeader[0x63] //N 
					, pHeader[0x64], pHeader[0x65] //
					, pHeader[0x66], pHeader[0x67] //
					, pHeader[0x6c], pHeader[0x6d] //Jn
					, pHeader[0x6e], pHeader[0x6f] //Jn
					);
#endif
			StringCchCat (szTitleNameFile, _countof (szTitleNameFile), szTemp);
		}

		//t@C̍쐬
		FileIndex = 1;
		StringCchPrintf (pData->m_szFileName, _countof (pData->m_szFileName), _T("%s.mpg"), szTitleNameFile);
		StringCchPrintf (szTemp, _countof (szTemp), _T("%s%s"), m_szSaveFolder, pData->m_szFileName);
		while (FileIndex < 100)//Ƃ肠100܂ł̓gC
		{
			hFile = ::CreateFile (szTemp, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_NEW, 0, NULL);
			if (hFile != INVALID_HANDLE_VALUE)
			{	break;
			}
			pData->m_LastError = ::GetLastError ();

			StringCchPrintf (pData->m_szFileName, _countof (pData->m_szFileName), _T("%s_%u.mpg"), szTitleNameFile, FileIndex++);
			StringCchPrintf (szTemp, _countof (szTemp), _T("%s%s"), m_szSaveFolder, pData->m_szFileName);
		}

		if (hFile == INVALID_HANDLE_VALUE)
		{
			Callback (TYPE_STATUS_ERROR, pData);
			DestroyDataConnection (pCfg);
			SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");
			break;
		}

		Callback (TYPE_FILE_CREATE_SUCCESS, pData);

		//wb_[̏
		if (!::WriteFile (hFile, pHeader, HeaderLength, &BytesWritten, NULL))
		{
			pData->m_LastError = ::GetLastError ();
			Callback (TYPE_STATUS_ERROR, pData);
			DestroyDataConnection (pCfg);
			SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");
			break;
		}
		pData->m_WriteLength += HeaderLength;

		//ԑgeLXg̏o
		if (m_bMakeInfoTxt)
		{
			HANDLE hTextFile = INVALID_HANDLE_VALUE;
			CHAR szText[1024] = {0};
			CHAR szFormat[1024] = {0};
			size_t TextLen = 0;

			if (::LoadStringA (GetInstanceHandle (), IDS_INFOTEXT, szFormat, _countof (szFormat)))
			{
				StringCchPrintfA (szText, _countof (szText), szFormat, szTitleName, szTitleInfo
					, pHeader[0x62], pHeader[0x63] //N 
					, pHeader[0x64], pHeader[0x65] //
					, pHeader[0x66], pHeader[0x67] //
					, pHeader[0x6c], pHeader[0x6d] //Jn
					, pHeader[0x6e], pHeader[0x6f] //Jn
					, pHeader[0x70], pHeader[0x71] //I
					, pHeader[0x72], pHeader[0x73] //I
					, szTitleChannel);

				StringCchPrintf (szTemp, _countof (szTemp), _T("%s%s"), m_szSaveFolder, pData->m_szFileName);
				StringCchCat (szTemp, _countof (szTemp), _T(".txt"));

				hTextFile = CreateFile (szTemp, GENERIC_WRITE, FILE_SHARE_READ, NULL, CREATE_NEW, 0, NULL);
				if (hTextFile != INVALID_HANDLE_VALUE)
				{	
					if (SUCCEEDED (StringCbLengthA (szText, sizeof (szText), &TextLen)))
					{	
						::WriteFile (hTextFile, szText, (ULONG)TextLen, &BytesWritten, NULL);
					}
					::CloseHandle (hTextFile);
				}
			}
		}
		
		DBG_MSG(("ERdFtpServer::DubbingDatSTOR - LOOP pCfg=%p \r\n", pCfg));

		//M[v
		while (bLoop)
		{
			bLoop = Receive (pCfg, pCfg->m_Data.m_hSocket, pCfg->m_Data.m_hEvent, pBuffer, BufferLength, &RecvLength, RECEIVE_TIME_OUT);

			if (RecvLength != 0)
			{
				if (!::WriteFile (hFile, pBuffer, RecvLength, &BytesWritten, NULL))
				{
					DBG_MSG(("ERdFtpServer::DubbingDatSTOR - WriteFile failure, pCfg=%p \r\n", pCfg));

					pData->m_LastError = ::GetLastError ();
					Callback (TYPE_STATUS_ERROR, pData);
					DestroyDataConnection (pCfg);
					SendResponse (pCfg, "426 Connection closed; transfer aborted.\r\n");
					break;
				}

				pData->m_WriteLength += BytesWritten;
			}

			if ((::GetTickCount () - pData->m_sTime) > 1000)
			{
				Callback (TYPE_STATUS_UPDATE, pData);
			}
		}
		DBG_MSG(("ERdFtpServer::DubbingDatSTOR - LOOP end, pCfg=%p, bLoop=%u \r\n", pCfg, bLoop));
		
		Callback (TYPE_STATUS_UPDATE, pData);
		Callback (TYPE_RECEIVE_COMPLETE, pData);
	
		DestroyDataConnection (pCfg);

		SendResponse (pCfg, "226 Transfer complete.\r\n");

	}
	while (0);

	if (hFile != INVALID_HANDLE_VALUE)
	{
		::CloseHandle (hFile);
	}

	if (pBuffer != NULL)
	{
		delete pBuffer;
	}
}

VOID ERdFtpServer::DubbingStatusXmlSTOR (EFtpClientCfg* pCfg)
{
}

VOID ERdFtpServer::DubbingCommandXmlSTOR (EFtpClientCfg* pCfg)
{
}
