/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#pragma once
#pragma push_macro("new")

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

template<class TYPE>
class EStrConv
{
public:
	EStrConv ()
	{
		m_psz = NULL;
	}

	~EStrConv ()
	{
		Free ();
	}

	VOID Free ()
	{
		if (m_psz != NULL)
		{
			delete m_psz;
			m_psz = NULL;
		}
	}

	TYPE Detach ()
	{
		TYPE p = m_psz;
		m_psz = NULL;
		return p;
	}

	operator TYPE () const
	{
		return m_psz;
	}

protected:
	TYPE	m_psz;
};

class EX2W : public EStrConv<LPWSTR>
{
public:
	EX2W (LPCWSTR psz = NULL)
	{	
		Set (psz);
	}

	EX2W (LPCSTR psz)
	{
		Set (psz);
	}

	VOID Set (LPCSTR psz)
	{
		int Length;

		Free ();

		if (psz == NULL)
		{	return;
		}

		Length = ::MultiByteToWideChar (CP_ACP, 0, psz, -1, NULL, 0);
		if (Length == 0)
		{	return;
		}

		m_psz = new WCHAR[Length];
		if (m_psz == NULL)
		{	return;
		}

		::MultiByteToWideChar (CP_ACP, 0, psz, -1, m_psz, Length);
	}

	VOID Set (LPCWSTR psz)
	{
		size_t Length = 0;

		Free ();

		if (psz == NULL)
		{	return;
		}

		if (FAILED (StringCchLengthW (psz, STRSAFE_MAX_CCH, &Length)))
		{	return;
		}

		m_psz = new WCHAR[Length + 1];
		if (m_psz == NULL)
		{	return;
		}

		StringCchCopyW (m_psz, Length + 1, psz);
	}
};


class EX2A : public EStrConv<LPSTR>
{
public:

	EX2A (LPCSTR psz = NULL)
	{	
		Set (psz);
	}

	EX2A (LPCWSTR psz)
	{
		Set (psz);
	}

	VOID Set (LPCWSTR psz)
	{
		int Length;

		Free ();

		if (psz == NULL)
		{	return;
		}

		Length = ::WideCharToMultiByte (CP_ACP, 0, psz, -1, NULL, 0, NULL, NULL);
		if (Length == 0)
		{	return;
		}

		m_psz = new CHAR[Length];
		if (m_psz == NULL)
		{	return;
		}

		::WideCharToMultiByte (CP_ACP, 0, psz, -1, m_psz, Length, NULL, NULL);
	}

	VOID Set (LPCSTR psz)
	{
		size_t Length = 0;

		Free ();

		if (psz == NULL)
		{	return;
		}

		if (FAILED (StringCchLengthA (psz, STRSAFE_MAX_CCH, &Length)))
		{	return;
		}

		m_psz = new CHAR[Length + 1];
		if (m_psz == NULL)
		{	return;
		}

		StringCchCopyA (m_psz, Length + 1, psz);
	}
};

class EX2X
{
public:
	EX2X ()
	{
	}

	EX2X (LPCSTR psz)
	{
		Set (psz);
	}

	EX2X (LPWSTR psz)
	{
		Set (psz);
	}

	VOID Set (LPCSTR psz)
	{
		m_W.Set (psz);
		m_A.Set (psz);
	}

	VOID Set (LPCWSTR psz)
	{
		m_A.Set (psz);
		m_W.Set (psz);
	}

	operator LPSTR () const
	{
		return m_A;
	}

	operator LPWSTR () const
	{
		return m_W;
	}

protected:
	EX2A	m_A;
	EX2W	m_W;
};

#define EW2W EX2W
#define EA2W EX2W
#define EW2A EX2A
#define EA2A EX2A

#ifdef _UNICODE
#define EA2T EX2W
#define EW2T EX2W
#define ET2A EX2A
#define ET2W EX2W
#else
#define EA2T EX2A
#define EW2T EX2A
#define ET2A EX2A
#define ET2W EX2W
#endif


#pragma pop_macro("new")
