/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "StdAfx.h"
#include "EWinPcap.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

EWinPcap::EWinPcap(void)
{
	m_hStop				= ::CreateEvent (NULL, TRUE, FALSE, NULL);
	m_DevList			= NULL;
	m_nThreadRunning	= 0;

	pcap_findalldevs	= NULL;
	pcap_freealldevs	= NULL;
	pcap_open_live		= NULL;
	pcap_close			= NULL;
	pcap_next_ex		= NULL;
	pcap_setfilter		= NULL;
	pcap_compile		= NULL;
	pcap_freecode		= NULL;
	pcap_sendpacket		= NULL;

	m_hWPcapDll = ::LoadLibrary (_T("wpcap.dll"));

	if (m_hWPcapDll != NULL)
	{
		*((FARPROC*)&pcap_findalldevs)	= ::GetProcAddress (m_hWPcapDll, "pcap_findalldevs");
		*((FARPROC*)&pcap_freealldevs)	= ::GetProcAddress (m_hWPcapDll, "pcap_freealldevs");
		*((FARPROC*)&pcap_open_live)	= ::GetProcAddress (m_hWPcapDll, "pcap_open_live");
		*((FARPROC*)&pcap_close)		= ::GetProcAddress (m_hWPcapDll, "pcap_close");
		*((FARPROC*)&pcap_next_ex)		= ::GetProcAddress (m_hWPcapDll, "pcap_next_ex");
		*((FARPROC*)&pcap_setfilter)	= ::GetProcAddress (m_hWPcapDll, "pcap_setfilter");
		*((FARPROC*)&pcap_compile)		= ::GetProcAddress (m_hWPcapDll, "pcap_compile");
		*((FARPROC*)&pcap_freecode)		= ::GetProcAddress (m_hWPcapDll, "pcap_freecode");
		*((FARPROC*)&pcap_sendpacket)	= ::GetProcAddress (m_hWPcapDll, "pcap_sendpacket");

		if ((pcap_findalldevs	== NULL) ||
			(pcap_freealldevs	== NULL) ||
			(pcap_open_live		== NULL) ||
			(pcap_close			== NULL) ||
			(pcap_next_ex		== NULL) ||
			(pcap_setfilter		== NULL) ||
			(pcap_compile		== NULL) ||
			(pcap_freecode		== NULL) ||
			(pcap_sendpacket	== NULL)
			)
		{
			::FreeLibrary (m_hWPcapDll);
			m_hWPcapDll		= NULL;
			pcap_findalldevs	= NULL;
			pcap_freealldevs	= NULL;
			pcap_open_live		= NULL;
			pcap_close			= NULL;
			pcap_next_ex		= NULL;
			pcap_setfilter		= NULL;
			pcap_compile		= NULL;
			pcap_freecode		= NULL;
			pcap_sendpacket		= NULL;
		}
	}
	
	m_szError[0] = 0;


}

EWinPcap::~EWinPcap(void)
{
	StopCapture ();

	if (m_hWPcapDll != NULL)
	{
		::FreeLibrary (m_hWPcapDll);
		m_hWPcapDll = NULL;
	}

	if (m_hStop != NULL)
	{
		::CloseHandle (m_hStop);
		m_hStop = NULL;
	}
}

LPCSTR EWinPcap::GetLastErrorString ()
{
	return m_szError;
}


BOOL EWinPcap::FindMacAddress (LPCSTR pszDevice, PUCHAR pMacAddress, size_t Size)
{
	ULONG				AdapterInfoSize		= 0;
	ULONG				InterfaceInfoSize	= 0;
	PIP_ADAPTER_INFO	pInfoBuffer			= NULL;
	PIP_ADAPTER_INFO	pInfo				= NULL;
	BOOL				RetCode				= FALSE;

	::ZeroMemory (pMacAddress, Size);

	do
	{
		if (::GetAdaptersInfo  (NULL, &AdapterInfoSize) != ERROR_BUFFER_OVERFLOW)
		{	break;
		}
	
		pInfoBuffer = (PIP_ADAPTER_INFO)new UCHAR [AdapterInfoSize];
		if (pInfoBuffer == NULL)
		{	break;
		}

		if (::GetAdaptersInfo  (pInfoBuffer, &AdapterInfoSize) != NO_ERROR)
		{
			break;
		}

		pInfo = pInfoBuffer;
		
		while (pInfo)
		{	
			if (StringCchStrStrA (pszDevice, STRSAFE_MAX_CCH, pInfo->AdapterName) != NULL)
			{
				memcpy (pMacAddress, pInfo->Address, min (pInfo->AddressLength, Size));
				RetCode = TRUE;
				break;
			}

			pInfo = pInfo->Next;
		}
	}
	while (0);

	if (pInfoBuffer != NULL)
	{	delete pInfoBuffer;
	}

	return RetCode;
}

BOOL EWinPcap::SetFilter(HANDLE hDevice, LPCSTR pszFilter)
{
	CBpfProgram		BpfProgram;
	BOOL			RetCode		= FALSE;

	do
	{
		if ((pcap_compile == NULL) || (pcap_setfilter == NULL))
		{	break;
		}

		if (hDevice == NULL)
		{	break;
		}

		if (pcap_compile(hDevice, &BpfProgram, pszFilter, 1, 0) < 0)
		{
			break;
		}

		if (pcap_setfilter (hDevice, &BpfProgram) < 0)
		{
			break;
		}

		RetCode = TRUE;
	}
	while (0);

	if (BpfProgram.m_pBpfInsns != NULL)
	{
		pcap_freecode (&BpfProgram);
	}

	return RetCode;
}

BOOL EWinPcap::IsRunning ()
{
	return (m_nThreadRunning > 0);
}

BOOL EWinPcap::StartCapture (LPCSTR pszFilter)
{
	CThread*			pThread		= NULL;
	ULONG				ThreadId	= 0;
	HANDLE				hDevice		= NULL;
	CPcapInterface*		pIf			= NULL;
	ULONG				Address		= 0;
	CPcapAddress*		pAddress	= NULL;

	if ((pcap_open_live == NULL) || (pcap_findalldevs == NULL) || (pcap_freealldevs == NULL))
	{	return FALSE;
	}

	StopCapture ();

	if (pcap_findalldevs (&m_DevList, m_szError) == -1)
	{	return FALSE;
	}

	if (m_DevList == NULL)
	{	return FALSE;
	}

	for (pIf = m_DevList; pIf != NULL; pIf = pIf->m_pNext)
	{
		hDevice = pcap_open_live (pIf->m_pszDeviceName, 65536, 0, 100, m_szError);
		if (hDevice == NULL)
		{	continue;
		}



		if (pszFilter != NULL)
		{
			SetFilter (hDevice, pszFilter);
		}


		pAddress = pIf->m_pAddresses;
		Address = 0;

		while (pAddress)
		{
			if (pAddress->m_pAddress->m_Family == AF_INET)
			{	
				Address = ((PSOCKADDR_IN)pAddress->m_pAddress)->sin_addr.S_un.S_addr;
				break;
			}
			pAddress = pAddress->m_pNext;
		}

		//if (pAddress == NULL)
		//{	continue;
		//}

		pThread = new CThread;
		if (pThread == NULL)
		{
			pcap_close (hDevice);
			continue;
		}

		ULONG PhyAddrLen = sizeof (pThread->m_pMacAddress);
		if ((Address == 0) || (::SendARP (Address, 0, (PULONG)pThread->m_pMacAddress, &PhyAddrLen) != 0))
		{
			if (!FindMacAddress (pIf->m_pszDeviceName, pThread->m_pMacAddress, sizeof (pThread->m_pMacAddress)))
			{
				delete pThread;
				pcap_close (hDevice);
				continue;
			}
		}

		pThread->m_pPcapInterface	= pIf;
		pThread->m_pWinPcap			= this;
		pThread->m_hDevice			= hDevice;
		pThread->m_pRunning			= &m_nThreadRunning;
		pThread->m_hThread			= ::CreateThread (NULL, 0, CaptureThread, pThread, 0, &ThreadId);

		if (pThread->m_hThread == NULL)
		{
			delete pThread;
			continue;
		}

		m_ThreadList.AddTail (pThread);
	}

	return !m_ThreadList.IsEmpty ();
}

void EWinPcap::StopCapture()
{
	CThread*		pThread;

	if ((pcap_close == NULL) || (pcap_freealldevs == NULL))
	{	return;
	}

	::SetEvent (m_hStop);
	::Sleep (100);

	while (!m_ThreadList.IsEmpty ())
	{
		pThread = m_ThreadList.RemoveHead ();
	
		if (pThread->m_hThread != NULL)
		{
			if (::WaitForSingleObject (pThread->m_hThread, 1000) == WAIT_TIMEOUT)
			{
				::TerminateThread (pThread->m_hThread, 0);
				DBG_MSG(("EWinPcap::StopCapture - WAIT_TIMEOUT (%08X, %s)\n", pThread->m_hDevice, pThread->m_pPcapInterface->m_pszDeviceName));
			}
		}

		delete pThread;
	}

	if (m_DevList != NULL)
	{
		pcap_freealldevs (m_DevList);
		m_DevList = NULL;
	}

	::ResetEvent (m_hStop);
}


ULONG EWinPcap::CaptureThread (PVOID pVoid)
{
	CThread*			pThread = (CThread*)pVoid;
	EWinPcap*			pThis	= pThread->m_pWinPcap;
	CPcapPacketHeader*	pHeader;
	const UCHAR*		pBuffer;
	int					Result;

	::InterlockedIncrement (pThread->m_pRunning);

	DBG_MSG(("EWinPcap::CaptureThread - START(%08X, %s)\n", pThread->m_hDevice, pThread->m_pPcapInterface->m_pszDeviceName));

	while (::WaitForSingleObject (pThis->m_hStop, 0) == WAIT_TIMEOUT)
	{
		Result = pThis->pcap_next_ex (pThread->m_hDevice, &pHeader, &pBuffer);

		if (Result < 0)
		{	break;
		}
		if (Result != 1)
		{	continue;
		}

		pThis->OnReceive (pBuffer, pHeader->m_CaptureLength, pThread);
	}


	pThis->pcap_close (pThread->m_hDevice);
	pThread->m_hDevice = NULL;


	::InterlockedDecrement (pThread->m_pRunning);
	::CloseHandle (pThread->m_hThread);
	pThread->m_hThread = NULL;
	
	DBG_MSG(("EWinPcap::CaptureThread - STOP(%s)\n", pThread->m_pPcapInterface->m_pszDeviceName));
	

	return 0;
}

VOID EWinPcap::OnReceive (const UCHAR* pBuffer, ULONG Length, CThread* pThread)
{
}
