/*---------------------------------------------------------------------------*/
//       Author : HDBENCH.NET
//         Mail : eword@hdbench.net
//          Web : http://hdbench.net/
//      License : The modified BSD license
//
//                           Copyright 2008 HDBENCH.NET, All rights reserved.
/*---------------------------------------------------------------------------*/

#include "StdAfx.h"
#include "EWindow.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

UINT EWindow::WM_DELETE_THIS	= ::RegisterWindowMessage (_T("EWindow::WM_DELETE_THIS"));

EList<EWindow*>		EWindow::s_HookWindowList;
HHOOK				EWindow::s_hHook				= NULL;
LONG				EWindow::s_HookCount			= 0;

EWindow::EWindow(void)
{
	m_hWnd			= NULL;
	m_pfnDefWndProc	= NULL;

	Initialize ();
}

EWindow::~EWindow(void)
{
	ASSERT (m_hWnd == NULL);

	DeInitialize ();
}


LRESULT CALLBACK EWindow::CBTProc (int Code, WPARAM wParam, LPARAM lParam)
{
	LPCBT_CREATEWND	pCreateWnd	= (LPCBT_CREATEWND)lParam;
	EWindow*		pWindow		= NULL;
	EWindow*		pWindowTmp	= NULL;
	HWND			hWnd		= (HWND)wParam;
	BOOL			fFound		= FALSE;
	HANDLE			hList		= NULL;
	
	if ((Code != HCBT_CREATEWND) || (pCreateWnd->lpcs == NULL))
	{
		return ::CallNextHookEx (s_hHook, Code, wParam, lParam);
	}

	pWindow	= (EWindow*)pCreateWnd->lpcs->lpCreateParams;

	if (pWindow == NULL)
	{
		return ::CallNextHookEx (s_hHook, Code, wParam, lParam);
	}

	s_HookWindowList.Lock ();

	hList =	s_HookWindowList.GetHeadHandle ();
	while (hList)
	{

		pWindowTmp = s_HookWindowList.GetNext (hList);
		if (pWindowTmp == pWindow)
		{
			fFound = TRUE;
			break;
		}
	}

	s_HookWindowList.Unlock ();

	if (fFound)
	{
		WNDPROC WndProc;

		pWindow->m_hWnd = hWnd;
		WndProc = (WNDPROC)(LONG_PTR)::GetWindowLongPtr (hWnd, GWLP_WNDPROC);

		if ((WndProc != NULL) && (WndProc != pWindow->WndProc))
		{
			pWindow->m_pfnDefWndProc = WndProc;
#pragma warning (disable:4244)
			::SetWindowLongPtr (hWnd, GWLP_WNDPROC, (LONG_PTR)pWindow->WndProc);
#pragma warning (default:4244)
		}

		::SetLastError (0);
#pragma warning (disable:4244)
		if (!::SetWindowLongPtr (hWnd, GWLP_USERDATA, (LONG_PTR)pWindow))
#pragma warning (default:4244)
		{
			if (::GetLastError () != ERROR_SUCCESS)
			{
				ASSERT (FALSE);
				::DestroyWindow (hWnd);
			}
		}
	}

	return ::CallNextHookEx (s_hHook, Code, wParam, lParam);
}

VOID EWindow::Initialize ()
{
	::InterlockedIncrement (&s_HookCount);

	if (s_hHook != NULL)
	{	return;
	}

	s_hHook = ::SetWindowsHookEx (WH_CBT, CBTProc, NULL, ::GetCurrentThreadId());
}


VOID EWindow::DeInitialize ()
{
	if (::InterlockedDecrement (&s_HookCount) == 0)
	{
		::UnhookWindowsHookEx (s_hHook);
		s_hHook = NULL;
	}
}


BOOL EWindow::HookWindowCreate (EWindow* pWindow)
{
	BOOL RetCode = FALSE;

	s_HookWindowList.Lock ();
	s_HookWindowList.AddTail (pWindow);
	s_HookWindowList.Unlock ();

	return TRUE;
}

BOOL EWindow::UnHookWindowCreate (EWindow* pWindow)
{
	BOOL		RetCode		= FALSE;
	HANDLE		hList		= NULL;
	HANDLE		hListNext	= NULL;
	EWindow*	pWindowTmp	= NULL;

	s_HookWindowList.Lock ();

	hList =	s_HookWindowList.GetHeadHandle ();
	while (hList)
	{
		hListNext = hList;
		pWindowTmp = s_HookWindowList.GetNext (hListNext);
		if (pWindowTmp == pWindow)
		{
			s_HookWindowList.RemoveAt (hList);
			break;
		}
		hList = hListNext;
	}
	s_HookWindowList.Unlock ();

	return RetCode;
}

BOOL EWindow::CreateEx(ULONG ExStyle, LPCTSTR lpClassName, LPCTSTR lpWindowName, ULONG Style, int x, int y, int Width, int Height, HWND hWndParent, HMENU hMenu)
{
	ASSERT (m_hWnd == NULL);

	HookWindowCreate (this);

	m_hWnd = ::CreateWindowEx (ExStyle, lpClassName, lpWindowName, Style, x, y, Width, Height, hWndParent, hMenu, GetInstanceHandle (), this);

	UnHookWindowCreate(this);


	return (m_hWnd != NULL);
}

LRESULT EWindow::DefWndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	if (m_pfnDefWndProc != NULL)
	{	return m_pfnDefWndProc (hWnd, message, wParam, lParam);
	}
	return ::DefWindowProc (hWnd, message, wParam, lParam);
}


LRESULT EWindow::WndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
	EWindow* pThis = (EWindow*)(LONG_PTR)::GetWindowLongPtr (hWnd, GWLP_USERDATA);
	LRESULT Result = 0;

	if (hWnd == NULL)
	{
		ASSERT (FALSE);

		return ::DefWindowProc (hWnd, message, wParam, lParam);
	}

	if (pThis == NULL)
	{
		ASSERT (FALSE);

		return ::DefWindowProc (hWnd, message, wParam, lParam);
	}

	if (message == WM_DELETE_THIS)
	{
		::DestroyWindow (hWnd);
		delete pThis;

		return 1;
	}
	
	if (pThis->OnWndProc (hWnd, message, wParam, lParam, &Result))
	{
		return Result;
	}


	switch (message)
	{
	case WM_CREATE:
		{
			if (!pThis->OnCreate ((CREATESTRUCT*)lParam))
			{
				::DestroyWindow (hWnd);
				break;
			}
			
			return pThis->DefWndProc (hWnd, message, wParam, lParam);
		}
		break;

	case WM_SIZE:
		{
			pThis->OnSize ((UINT)wParam, LOWORD (lParam), HIWORD (lParam));

			return pThis->DefWndProc (hWnd, message, wParam, lParam);
		}
		break;

	case WM_NOTIFY:
		{
			NMHDR* pNmHdr = (NMHDR*)lParam;
			if (::IsWindow (pNmHdr->hwndFrom))
			{
				::SendMessage (pNmHdr->hwndFrom, WM_NOTIFY + WM_REFLECT_BASE, wParam, lParam);
			}

			//DBG_MSG(("EWindow::WndProc - hWnd=%08X, wParam=%08X, code=%08X, hwndFrom=%08X, idFrom=%08X \n", hWnd, wParam, pNmHdr->code, pNmHdr->hwndFrom, pNmHdr->idFrom));
			
			if (!pThis->OnNotify (wParam, lParam, &Result))
			{
				return pThis->DefWndProc (hWnd, message, wParam, lParam);
			}
		}
		break;

	case WM_NOTIFY + WM_REFLECT_BASE:
		{
			return pThis->OnNotifyRefrection (wParam, lParam, &Result);
		}
		break;

	case WM_COMMAND:
		{
			if (::IsWindow ((HWND) lParam))
			{
				::SendMessage ((HWND) lParam, WM_COMMAND + WM_REFLECT_BASE, wParam, lParam);
			}

			if (!pThis->OnCommand (wParam, lParam))
			{
				return pThis->DefWndProc (hWnd, message, wParam, lParam);
			}
		}
		break;

	case WM_COMMAND + WM_REFLECT_BASE:
		{
			return pThis->OnCommandRefrection (wParam, lParam);
		}
		break;

	case WM_PAINT:
		{
			if (!pThis->OnPaint ())
			{
				return pThis->DefWndProc (hWnd, message, wParam, lParam);
			}
		}
		break;

	case WM_DRAWITEM:
		{
			DRAWITEMSTRUCT* pDrawItemStruct = (DRAWITEMSTRUCT*)lParam;

			if (!::SendMessage (pDrawItemStruct->hwndItem, WM_DRAWITEM + WM_REFLECT_BASE, wParam, lParam))
			{
				return pThis->DefWndProc (hWnd, message, wParam, lParam);
			}
		}
		break;

	case WM_DRAWITEM + WM_REFLECT_BASE:
		{
			DRAWITEMSTRUCT* pDrawItemStruct = (DRAWITEMSTRUCT*)lParam;

			return pThis->OnDrawItemRefrection (wParam, pDrawItemStruct);
		}
		break;

	case WM_DESTROY:
		{
			pThis->OnDestroy ();
			return pThis->DefWndProc (hWnd, message, wParam, lParam);
		}
		break;

	case WM_NCDESTROY:
		{
			pThis->OnNcDestroy ();
			
			pThis->m_hWnd = NULL;

			Result = pThis->DefWndProc (hWnd, message, wParam, lParam);

			::SetLastError (0);
	#pragma warning (disable:4244)
			if (!::SetWindowLongPtr (hWnd, GWLP_USERDATA, 0))
	#pragma warning (default:4244)
			{
				if (::GetLastError () != ERROR_SUCCESS)
				{
					ASSERT (FALSE);
				}
			}

			if (pThis->m_pfnDefWndProc != NULL)
			{
	#pragma warning (disable:4244)
				::SetWindowLongPtr (hWnd, GWLP_WNDPROC, (LONG_PTR)pThis->m_pfnDefWndProc);
	#pragma warning (default:4244)

				pThis->m_pfnDefWndProc = NULL;
			}
		}
		break;


	default:
		{
			return pThis->DefWndProc (hWnd, message, wParam, lParam);
		}
		break;
	}

	return Result;
}

BOOL EWindow::OnWndProc (HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	return FALSE;
}




BOOL EWindow::OnCreate (CREATESTRUCT* pStruct)
{
	return TRUE;
}



VOID EWindow::OnDestroy ()
{
}

VOID EWindow::OnNcDestroy ()
{
}

VOID EWindow::OnSize (UINT nType, int cx, int cy)
{
}

BOOL EWindow::OnNotify (WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	return FALSE;
}

BOOL EWindow::OnNotifyRefrection (WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	return FALSE;
}

BOOL EWindow::OnCommandRefrection (WPARAM wParam, LPARAM lParam)
{
	return FALSE;
}

BOOL EWindow::OnPaint ()
{
/*	PAINTSTRUCT ps;
	HDC hdc;

	hdc = ::BeginPaint (m_hWnd, &ps);
	// TODO: Add any drawing code here...
	::EndPaint (m_hWnd, &ps);
*/
	return FALSE;
}

BOOL EWindow::OnDrawItemRefrection (WPARAM CtrlId,  DRAWITEMSTRUCT* pDrawItemStruct)
{
	return FALSE;
}

BOOL EWindow::OnCommand (WPARAM wParam, LPARAM lParam)
{
	return FALSE;
}

